/*
	$Id $

	Copyright (C) 2013 Nicklas Nordborg
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions;

import java.util.Iterator;

import net.sf.basedb.util.extensions.Action;

/**
	Abstract base class intended to be extended by {@link Action} implementations
	that want to support dynamic action attributes. Typically, if the action factory
	descends from {@link AbstractJspFactory}. 
	
	@author nicklas
	@since 3.3
*/
public abstract class DynamicActionAttributeSupport 
	implements DynamicActionAttributes 
{

	private Iterator<DynamicActionAttribute> dynamicAttributes;
	private DynamicActionAttributes source;
	
	/**
		If the object implements DynamicActionAttributes interface,
		get a string with all dynamic attributes, otherwise an empty string.
	*/
	public static String getAttributesString(Object action)
	{
		StringBuilder sb = new StringBuilder();
		if (action instanceof DynamicActionAttributes)
		{
			Iterator<DynamicActionAttribute> it = ((DynamicActionAttributes)action).getDynamicActionAttributes();
			if (it != null)	addAttributes(sb, it);
		}
		return sb.toString();
	}

	/**
		Append all dynamic action attributes to the string builder.
	*/
	public static void addAttributes(StringBuilder sb, Iterator<DynamicActionAttribute> it)
	{
		while (it.hasNext())
		{
			DynamicActionAttribute attr = it.next();
			sb.append(" ").append(attr.getName()).append("=\"").append(attr.getValue()).append("\"");
		}		
	}

	
	protected DynamicActionAttributeSupport()
	{}
	
	/*
		From the DynamicActionAttributes interface
		------------------------------------------
	*/
	@Override
	public Iterator<DynamicActionAttribute> getDynamicActionAttributes() 
	{
		Iterator<DynamicActionAttribute> it = null;
		if (source != null)
		{
			it = source.getDynamicActionAttributes();
		}
		return it;
	}
	// -------------------------------------------
	
	/**
		Set the source of dynamic action attributes. The source is another
		instance of an object that implements the {@link DynamicActionAttributes}
		interface.
	*/
	public void setDynamicActionAttributesSource(DynamicActionAttributes source)
	{
		this.source = source;
	}

}
