/*
	$Id $

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib;

/**
	A special button tag that is used to display a "Calendar" button
	that opens a popup for selecting a date and time. The id of 
	a form control must be specified in the "textarea" attribute.
	
	@author nicklas
	@since 3.3
*/
public class CalendarButton 
	extends Button 
{

	private static final long serialVersionUID = 6346550752955859402L;

	public CalendarButton()
	{}
	
	
	@Override
	protected void initDefaultAttributes() 
	{
		super.setTitle("Calendar&hellip;");
		setImage("calendar.png");
		setTooltip("Select a date from a calendar");
		setDynamicAttribute(null, "data-auto-init", "calendar");
	}

	public void setTextarea(String textarea)
	{
		setDynamicAttribute(null, "data-textarea-id", textarea);
	}

	/**
		The title attribute now goes into the calendar
		popup dialog instead. The button's title is fixed
		at 'Calendar'.
	*/
	@Override
	public void setTitle(String title) 
	{
		setDynamicAttribute(null, "data-title", title);
	}
	
	/**
		Make sure that "auto-init" is added to the class
		attribute.
	*/
	@Override
	public String getFullClass(String... extraClasses) 
	{
		String fc = super.getFullClass(extraClasses);
		
		if (fc == null)
		{
			fc = "auto-init";
		}
		else if (!fc.contains("auto-init"))
		{
			fc += " auto-init";
		}
		return fc;
	}
	
}
