/*
  $Id: ItemSubtypeFileTypeData.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	This class hols information about which file types
	can be used together with item subtypes.
	
	@author Nicklas
	@since 3.0
	@see net.sf.basedb.core.ItemSubtypeFileType
	@see <a href="../../../../../../html/developer/api/data_api.platforms.html">Developer documentation: Experimental platforms and item subtypes</a>
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (fr, 21 okt 2011) $
	@hibernate.class table="`ItemSubtypeFileTypes`" lazy="true"
*/
public class ItemSubtypeFileTypeData
	extends BasicData
{
	
	private DataFileTypeData fileType;
	/**
		Get the file type.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`datafiletype_id`" not-null="true" unique-key="uniquefile"
	*/
	public DataFileTypeData getDataFileType()
	{
		return fileType;
	}
	public void setDataFileType(DataFileTypeData fileType)
	{
		this.fileType = fileType;
	}
	
	private ItemSubtypeData subtype;
	/**
		Get the item subtype.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`subtype_id`" not-null="true" unique-key="uniquefile"
	*/
	public ItemSubtypeData getItemSubtype()
	{
		return subtype;
	}
	public void setItemSubtype(ItemSubtypeData subtype)
	{
		this.subtype = subtype;
	}
	
	
	private boolean required;
	/**
		If a file of this type is required or not.
		@hibernate.property column="`required`" type="boolean" not-null="true"
	*/
	public boolean isRequired()
	{
		return required;
	}
	public void setRequired(boolean required)
	{
		this.required = required;
	}

	private boolean multiple;
	/**
		If multiple files of this type are allowed in a file set or not.
		@hibernate.property column="`allow_multiple`" type="boolean" not-null="true"
	*/
	public boolean getAllowMultiple()
	{
		return multiple;
	}
	public void setAllowMultiple(boolean multiple)
	{
		this.multiple = multiple;
	}

	
}
