/**
	$Id: EngineFactory.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.dbengine;

import org.hibernate.dialect.Dialect;
import org.hibernate.dialect.MySQLDialect;
import org.hibernate.dialect.PostgreSQLDialect;

/**
 	Creates <code>DbEngine</code> objects based on the Hibernate dialect.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (to, 11 sep 2008) $
*/
public class EngineFactory
{
	/**
		Create a new <code>DbEngine</code> object.
		@param dialect The Hibernate dialect
		@return A DbEngine or null, if no implementation exists for the specified dialect
	*/
	public static DbEngine createEngine(Dialect dialect)
	{
		DbEngine engine = null;
		if (dialect instanceof MySQLDialect)
		{
			engine = new MySQLEngine();
		}
		else if (dialect instanceof PostgreSQLDialect)
		{
			engine = new PostgresDbEngine();
		}
		else
		{
			engine = new DefaultDbEngine();
		}
		return engine;
	}
}
