/**
	$Id: MultiUpdateWork.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;


/**
	A JDBC Work implementation that executes multiple queries that
	doesn't return any result.

	@author nicklas
	@version 2.9
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (to, 11 sep 2008) $
*/
public class MultiUpdateWork
	implements JdbcWork<Object>
{

	private Collection<String> statements;
	
	/**
		Create a new work item. 
		@param statements The sql statements to execute
	 */
	public MultiUpdateWork(Collection<String> statements)
	{
		this.statements = statements;
	}
	
	/**
		From the Work interface
		-----------------------
	*/
	@Override
	public void execute(Connection connection) 
		throws SQLException
	{
		if (statements == null || statements.isEmpty()) return;
		Statement s = null;
		try
		{
			s = connection.createStatement();
			for (String sql : statements)
			{
				if (sql != null) s.executeUpdate(sql);
			}
		}
		finally
		{
			try
			{
				if (s != null) s.close();
			}
			catch (Throwable t)
			{}
		}
	}
	// --------------------------
	
	/*
		From the JdbcWork interface
		---------------------------
	*/
	/**
		@return Always null
	*/
	@Override
	public Object getResult()
	{
		return null;
	}
	// ----------------------------

}
