/**
	$Id: TableExistsWork.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
	A work implementation that checks if a given table exists 
	in the database or not.

	@author nicklas
	@version 2.9
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (to, 11 sep 2008) $
*/
public class TableExistsWork
	implements JdbcWork<Boolean>
{
	private final String catalog;
	private final String schema;
	private final String table;
	private boolean hasTable;
	
	/**
		Create a new work item.
		@param catalog The catalog the table is located in or null
			to ignore any catalog information
		@param schema The schema the table is located in or null
			to ignore any schema information
		@param table The table to check
	*/
	public TableExistsWork(String catalog, String schema, String table)
	{
		this.catalog = catalog;
		this.schema = schema;
		this.table = table;
	}
	
	/**
		From the Work interface
		-----------------------
	*/
	@Override
	public void execute(Connection connection) 
		throws SQLException
	{
		ResultSet tables = null;
		hasTable = false;
		try
		{
			DatabaseMetaData metaData = connection.getMetaData();
			tables = metaData.getTables(catalog, schema, table, null);
			hasTable = tables.next();
		}
		finally
		{
			try
			{
				if (tables != null) tables.close();
			}
			catch (Throwable t)
			{}
		}
	}
	// --------------------------
	
	/*
		From the JdbcWork interface
		---------------------------
	*/
	/**
		@return TRUE if the table exists, FALSE otherwise
	*/
	@Override
	public Boolean getResult()
	{
		return hasTable;
	}
	// ----------------------------

}
