/**
	$Id: TypeWrapper.java 5876 2011-11-15 13:45:50Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import org.hibernate.type.BooleanType;
import org.hibernate.type.DateType;
import org.hibernate.type.DoubleType;
import org.hibernate.type.FloatType;
import org.hibernate.type.IntegerType;
import org.hibernate.type.LongType;
import org.hibernate.type.ShortType;
import org.hibernate.type.StringType;
import org.hibernate.type.TextType;
import org.hibernate.type.TimestampType;

/**
	Wrapper class for Hibernate types that will try to
	"hide" issues with the redesign of the type system in Hibernate
	that is going on in Hibernate 3.5.2 --> Hibernate 3.6.
	
	@author Nicklas
	@since 2.16
	@base.modified $Date: 2011-11-15 14:45:50 +0100 (ti, 15 nov 2011) $
*/
public class TypeWrapper<T extends org.hibernate.type.Type>
{
	
	public static final TypeWrapper<ShortType> SHORT = 
		new TypeWrapper<ShortType>(ShortType.INSTANCE, 2);

	public static final TypeWrapper<IntegerType> INTEGER = 
		new TypeWrapper<IntegerType>(IntegerType.INSTANCE, 4);

	public static final TypeWrapper<LongType> LONG = 
		new TypeWrapper<LongType>(LongType.INSTANCE, 8);

	public static final TypeWrapper<FloatType> FLOAT = 
		new TypeWrapper<FloatType>(FloatType.INSTANCE, 4);

	public static final TypeWrapper<DoubleType> DOUBLE = 
		new TypeWrapper<DoubleType>(DoubleType.INSTANCE, 8);

	public static final TypeWrapper<StringType> STRING = 
		new TypeWrapper<StringType>(StringType.INSTANCE, 0);

	public static final TypeWrapper<TextType> TEXT = 
		new TypeWrapper<TextType>(TextType.INSTANCE, 0);

	public static final TypeWrapper<BooleanType> BOOLEAN = 
		new TypeWrapper<BooleanType>(BooleanType.INSTANCE, 1);
	
	public static final TypeWrapper<DateType> DATE = 
		new TypeWrapper<DateType>(DateType.INSTANCE, 4);

	public static final TypeWrapper<TimestampType> TIMESTAMP = 
		new TypeWrapper<TimestampType>(TimestampType.INSTANCE, 4);


	private final T hibernateType;
	private final int sqlType;
	private final int size;
	
	protected TypeWrapper(T hibernateType, int size)
	{
		this.hibernateType = hibernateType;
		this.sqlType = hibernateType instanceof org.hibernate.type.SingleColumnType ?
			((org.hibernate.type.SingleColumnType)hibernateType).sqlType() : 0;
		this.size = size;
	}
	
	/**
		Get the underlying Hibernate type.
	*/
	public T getHibernateType()
	{
		return hibernateType;
	}
	
	/**
		Get the SQL code for database columns that can store this type.
		@return A code, or 0 if not applicable to the given type
		@see java.sql.Types
	*/
	public int getSQLType()
	{
		return sqlType;
	}
	
	/**
		Get the number of bytes a single value for this type occupies 
		in a database (approximately). If the type represents a 
		variable-length property, 0 is returned.
		
		@return The number of bytes required in the database
			or 0 if not known
		@since 3.1
	*/
	public int getApproximateSize()
	{
		return size;
	}
	

}
