/**
	$Id: ChangeType.java 6510 2014-08-11 07:29:17Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.log;

/**
	Enumeration that representst the type of change that has happened.

	@author Nicklas
	@version 2.13
	@base.modified $Date: 2014-08-11 09:29:17 +0200 (må, 11 aug 2014) $
*/
public enum ChangeType
{

	CREATE(1, "created"),
	UPDATE(2, "updated"),
	DELETE(3, "deleted"),
	ADD(4, "added"),
	REMOVE(5, "removed");
	
	private final int value;
	private final String was;
	
	private ChangeType(int value, String was)
	{
		this.value = value;
		this.was = was;
	}
	
	/**
		Get the numerical value for this change type. Each change
		type has a unique value.
	*/
	public int getValue()
	{
		return value;
	}
	
	/**
		Create a string for display purposes that explains what
		has happened to the what. For example, if what is 'Annotation',
		this method may return 'Annotation created'.
		@param what The 'what' that something happened to
		@return A string
	*/
	public String was(String what)
	{
		return what + " " + was;
	}
	
	/**
		Get the change type corresponding to the given numerical value.
		@param value The numerical value
		@return The change type, or null if there is no change type with the
			same valye
	*/
	public static ChangeType fromValue(int value)
	{
		for (ChangeType ct : ChangeType.values())
		{
			if (ct.value == value) return ct;
		}
		return null;
	}
	
}
