/**
	$Id: OlderThanFileFilter.java 4827 2009-03-23 12:09:35Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.io.File;
import java.io.FileFilter;

/**
	An implementation of the {@link FileFilter} interface
	that filter files based on their age.

	@author nicklas
	@version 2.11
	@base.modified $Date: 2009-03-23 13:09:35 +0100 (må, 23 mar 2009) $
*/
public class OlderThanFileFilter
	implements FileFilter
{

	private final long maxAge;
	private final boolean recursive;
	
	/**
		Create a new file filter that matches files that are older than
		the specified age.
		@param maxAge The age limit (in milliseconds). Files that are 
			older are matched by this filter
		@param recursive If we should recurse into subdirectories 
			or not
	*/
	public OlderThanFileFilter(long maxAge, boolean recursive)
	{
		this.maxAge = maxAge;
		this.recursive = recursive;
	}
	
	/**
		Accept files that are older than the specified last
		modified time.
	*/
	@Override
	public boolean accept(File file)
	{
		if (file == null)
		{
			return false;
		}
		else if (file.isDirectory())
		{
			return recursive;
		}
		else
		{
			long lm = file.lastModified();
			long age = System.currentTimeMillis()-lm;
			return lm > 0 && age >= maxAge;
		}
	}

}
