/**
	$Id: CollectionTransformer.java 4723 2009-01-13 08:20:26Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.collections;

import java.util.Collection;


/**
	Allows a collection to be transformed by applying some operation to it. 
	The transformed items should be added to the destination collection. The
	source collection is not allowed to be modified. Client applications 
	should NOT use the same collection as both source and destination since that
	may interfere with iterators on the source collection. 
	The destination collection may contain elements of the same or a different type
	as the source collection.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2009-01-13 09:20:26 +0100 (ti, 13 jan 2009) $
	@param <D> The element type contained in the destination collection
	@param <S> The element type of the source collection
*/
public interface CollectionTransformer<S, D>
{
	/**
		Transform the source collection and store the changes in the
		destination collection.
		
		@param source The source collection
		@param destination The destination collection
		@return The number of elements affected in the destination collection. Note!
			This value may have a different meaning depending on the implementor. For
			example, some implementor may add elements while other implementor may 
			remove elements. A return value of 0 should always indicate that no modifications 
			has been made to the destination collection.
	*/
	public int transform(Collection<? extends S> source, Collection<? super D> destination);
	
}
