/**
	$Id: ErrorHandlerFactory.java 5486 2010-11-12 13:59:38Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

/**
	Error handler factories are responsible for creating error handlers.
	Each extension point may provide a specific factory implementation
	(see {@link ExtensionPoint#getErrorHandlerFactory()}). If no factory
	is given, a default error handler will be used by the system
	(eg. {@link LoggingErrorHandlerFactory}). The error handler factory
	must be thread-safe since it can be used by many threads at the same
	time.

	@author Nicklas
	@since 2.17
	@base.modified $Date: 2010-11-12 14:59:38 +0100 (fr, 12 nov 2010) $
*/
public interface ErrorHandlerFactory<A extends Action>
{

	/**
		Get an error handler for the current context. This method is called
		once for each request/use of an extension point. Errors that happen
		when processing things related to this extension point will be
		directed to the error handler returned by this method. The factory
		may create a new or it may re-use the same error handler for each 
		request. If the error handler is re-used it must be thread-safe.
		
		@param context The current invokation context which is an extension
			point context (the {@link InvokationContext#getExtension()} method
			is expected to return null)
	*/
	public ErrorHandler<A> getErrorHandler(InvokationContext<? extends A> context);
	
}
