/*
	$Id$

	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.jep;

import java.util.Stack;

import org.nfunk.jep.ParseException;

/**
	A simple base class for registering a JEP function by name and number
	of parameters. The default implementation will simply return a random
	number between 0 and 100. This is intended to be re-implemented by 
	subclasses.
	
	@author nicklas
	@since 3.3
*/
public class SimpleJepFunction 
	implements JepFunction 
{
	private final String name;
	private final int numParameters;
	private int actualParameters;
	
	public SimpleJepFunction(String name, int numParameters)
	{
		this.name = name;
		this.numParameters = numParameters;
	}
	
	/*
		From the JepFunction interface
		-------------------------------------------
	*/
	@Override
	public String getFunctionName()
	{
		return name;
	}
	// -------------------------------------------
	@Override
	public int getNumberOfParameters()
	{
		return numParameters;
	}

	@Override
	public boolean checkNumberOfParameters(int n)
	{
		return numParameters < 0 || n == numParameters;
	}

	@Override
	public void setCurNumberOfParameters(int actualParameters) 
	{
		this.actualParameters = actualParameters;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void run(Stack stack) 
		throws ParseException 
	{
		int parameters = numParameters < 0 ? actualParameters : numParameters;
		for (int i = 0; i < parameters; ++i)
		{
			stack.pop();
		}
		stack.push(Math.random()*100);
	}

}
