/**
	$Id: ConnectionManagerFactory.java 6493 2014-06-25 11:37:10Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.uri;

import java.net.URI;

import net.sf.basedb.util.extensions.Action;

/**
	A connection manager factory is responsible for creating connection
	managers for some subset of URI:s that is supported. A factory implementation
	must be thread-safe since it may be used by more than one thread at
	a time. 

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2014-06-25 13:37:10 +0200 (on, 25 jun 2014) $
*/
public interface ConnectionManagerFactory
	extends Action
{

	/**
		Get the ID of this connection manager factory. This should be
		a unique string matching the id of extension that has been 
		registered with BASE.
	*/
	public String getId();
	
	/**
		Get a short string that is suitable for use in selection lists, etc.
		in client applications.
		@return A string
	*/
	public String getDisplayName();
	
	/**
		Get a longer description that can be used by client applications to
		show more information about the capabilities of the factory to
		the users.
		@return A string or null if no more information is available
	*/
	public String getDescription();

	/**
		Does this factory support auto-detection if an URI is
		supported or not? 
	*/
	public boolean useAutoDetection();
	
	/**
		Check if the given factory knows how to create a handler 
		for the given URI.
		@param uri The URI to check
	*/
	public boolean supports(URI uri);
	
	/**
		Create a new handler for accessing the resource at
		the given URI. Unless otherwise noted it is expected that
		this method create a new connection manager for each call.
		<p>
		NOTE! Also consider if it is possible for the connection manager
		to implement the {@link ResumableConnectionManager} interface.
		This will improve support for resuming downloads at a specified
		location.
		
		@param uri The URI to access (required)
		@param parameters Connection parameters, such as user login/password, etc. 
			Optional, unless otherwise is noted by the implementation.
		@return An URI handler
	*/
	public ConnectionManager createConnectionManager(URI uri, ConnectionParameters parameters);
	
}
