
/*
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;

/**
	A simple program that expects a BFS metadata file
	as input. If a [files] section is found it will add
	a couple of extra file entries to it. 
	It accepts one parameter on the command line:
	
	nosdata: Remove the [sdata] section and a all file 
            entries in the files section.
	
	@author nicklas
	@since 2.15
*/
public class BfsExtraFileGenerator
{
	public static void main(String[] args) 
	{
		String cmd = args.length > 0 ? args[0] : null;
		boolean nosdata = "nosdata".equals(cmd);

		try
		{
			BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
			String line = in.readLine();
			boolean skipSection = false;
			
			while (line != null)
			{
				if (skipSection)
				{
					// Skip lines until a new section is found
					if (line.startsWith("["))
					{
						skipSection = false;
					}
					else
					{
						line = null;
					}
				}
				
				if ("[files]".equals(line))
				{
					// Print out extra files
					line =
						"[files]\n" +
						"x-file1\tfile1.txt\n" +
						"x-file2\tfile2.txt";
					skipSection = nosdata;
				}
				else if ("[sdata]".equals(line))
				{
					skipSection = nosdata;
					if (skipSection) line = null;
				}
				if (line != null) System.out.println(line);
				line = in.readLine();
			}
						
			createFile("file1.txt");
			createFile("file2.txt");
		}
		catch (Throwable t)
		{
			t.printStackTrace();
			System.exit(1);
		}
	}
	
	private static void createFile(String name)
		throws IOException
	{
		File f = new File(name);
		PrintWriter pw = new PrintWriter(f);
		for (int i = 0; i < 100; ++i)
		{
			pw.println(Math.random());
		}
		pw.close();
	}
	
}
