/*
	$Id: TestIlluminaImporter.java 6100 2012-09-04 08:43:21Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import java.util.ArrayList;
import java.util.List;

import net.sf.basedb.core.*;
import net.sf.basedb.core.plugin.*;

public class TestIlluminaImporter
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing Illumina rawdata import");
		// Test import rawdata without lims connection
		TestReporter.test_import_from_file("data/test.illumina.reporters.txt", "Search_key,Target,ProbeId.*", ",(?=(?:[^\"]*\"[^\"]*\")*(?![^\"]*\"))", 1, 1);
		int fileId = TestFile.test_create("data/test.illumina.rawdata.txt", false, false);
		int hyb1Id = TestPhysicalBioAssay.test_create(0, "Hyb #1677718123", 0, 12);
		int scan1Id = TestDerivedBioAssay.test_create_root("1677718123", hyb1Id, SystemItems.getId(DerivedBioAssay.SCAN), 0, 0, 0, 0);
		int hyb2Id = TestPhysicalBioAssay.test_create(0, "Hyb #1677718142", 0, 12);
		int scan2Id = TestDerivedBioAssay.test_create_root("1677718142", hyb2Id, SystemItems.getId(DerivedBioAssay.SCAN), 0, 0, 0, 0);
		int protocolId = TestProtocol.test_create(SystemItems.getId(Protocol.FEATURE_EXTRACTION), null, false);
		int softwareId = TestSoftware.test_create(SystemItems.getId(Software.FEATURE_EXTRACTION), null, false);
		int experimentId = TestExperiment.test_create("illumina", false);
		
		// Extracts
		int cy3 = TestTag.test_load("cy3");
		int labeledExtract = SystemItems.getId(Extract.LABELED);
		int[] extractIds = new int[12];
		for (int i = 0; i < 12; ++i)
		{
			extractIds[i] = TestExtract.test_create(0, "#" + (i+1), labeledExtract, cy3, false);
			TestPhysicalBioAssay.test_add_extract(hyb1Id, extractIds[i], i+1, null);
		}
		
		// Create plugin configuration and job
		int pluginDefinitionId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.IlluminaRawDataImporter");
		int jobId = test_create_job(pluginDefinitionId, fileId, protocolId, softwareId, experimentId, scan1Id, scan2Id);
		
		// Execute job
		ok = ok && TestJob.test_execute(jobId, false);
		
		// Print result
		int[] rawBioAssays = test_getRawBioAssays(experimentId);
		if (rawBioAssays.length > 0)
		{
			ok &= TestRawBioAssay.test_list_raw_data(rawBioAssays[0], 10, 5, 97, true);
		}
		
		// Clean up test
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestExperiment.test_delete(experimentId);	
		for (int rawId : rawBioAssays)
		{
			TestRawBioAssay.test_delete(rawId);
		}
		TestJob.test_delete(jobId);
		
		TestDerivedBioAssay.test_delete(scan1Id);
		TestDerivedBioAssay.test_delete(scan2Id);
		
		TestSoftware.test_delete(softwareId);
		TestProtocol.test_delete(protocolId);
		TestPhysicalBioAssay.test_delete(hyb1Id);
		TestPhysicalBioAssay.test_delete(hyb2Id);
		TestFile.test_delete(fileId);
		TestReporter.test_delete();
		for (int i = 0; i < extractIds.length; ++i)
		{
			TestExtract.test_delete(extractIds[i]);
		}

		write("++Testing Illumina rawdata import "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create_job(int pluginId, int fileId, int protocolId, int softwareId, int experimentId, int... scanIds)
	{
		if (pluginId == 0 || fileId == 0 || experimentId == 0 && !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			File file = File.getById(dc, fileId);

			PluginDefinition plugin = PluginDefinition.getById(dc, pluginId);
			Experiment experiment = (experimentId > 0) ? Experiment.getById(dc, experimentId) : null;
			Job j = Job.getNew(dc, plugin, null, experiment);
			j.setName("Import Illumina data from file " + file.getName());

			GuiContext context = new GuiContext(Item.EXPERIMENT, GuiContext.Type.LIST);
			PluginConfigurationRequest request = j.configure(context);
			write_request_information(request.getRequestInformation());
			request.setParameterValue("file", file);
			if (experimentId != 0) 
			{
				request.setParameterValue("experiment", Experiment.getById(dc, experimentId));
			}
			if (scanIds != null && scanIds.length > 0) 
			{
				List<DerivedBioAssay> bioAssays = new ArrayList<DerivedBioAssay>(scanIds.length);
				for (int scanId : scanIds)
				{
					bioAssays.add(DerivedBioAssay.getById(dc, scanId));
				}
				request.setParameterValues("bioAssays", bioAssays);
			}
			if (protocolId != 0) 
			{
				request.setParameterValue("protocol", Protocol.getById(dc, protocolId));
			}
			if (softwareId != 0) 
			{
				request.setParameterValue("software", Software.getById(dc, softwareId));
			}
			request.setParameterValue("decimalSeparator", "dot");
			request.setParameterValue("defaultError", "fail");
			request.setParameterValue("missingReporterError", "null");
			request.setParameterValue("invalidColumnsError", "ignore");
			request.setParameterValue("numberFormatError", "null");
			request.setParameterValue("featureMismatchError", "smart");

			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				dc.saveItem(j);
				response.saveParameters(dc);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for Illumina import OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for Illumina import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}

	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter<?> pp : ri.getParameters())
		{
			ParameterType pt = pp.getParameterType();
			System.out.println("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt );
		}
	}
	
	static int[] test_getRawBioAssays(int experimentId)
	{
		int[] rawId = new int[0];
		if (experimentId == 0) return rawId;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment experiment = Experiment.getById(dc, experimentId);
			List<RawBioAssay> rawBioAssays = experiment.getRawBioAssays().list(dc);
			rawId = new int[rawBioAssays.size()];
			
			for (int i = 0; i < rawId.length; ++i)
			{
				rawId[i] = rawBioAssays.get(i).getId();
			}
			write("--Get imported Illumina raw bioassays OK");
		}
		catch (Throwable ex)
		{
			write("--Get imported of Illumina raw bioassays FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return rawId;
	}
	
}

