/*
	$Id: TestItemImporter.java 6558 2014-10-08 08:41:39Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import java.util.Collections;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DerivedBioAssay;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.Software;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.Response;


public class TestItemImporter
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing item import using plugin");
		
		// Project
		int projectId = TestProject.test_create("Default project", false);
		TestProject.test_activate_project(projectId, null);
		
		// Create referenced items
		int samplingProtocolId = TestProtocol.test_create(SystemItems.getId(Protocol.SAMPLING), "Sampling", false);
		int defaultSamplingProtocolId =  TestProtocol.test_create(SystemItems.getId(Protocol.SAMPLING), "Sampling.def", false);
		int extractionProtocolId = TestProtocol.test_create(SystemItems.getId(Protocol.EXTRACTION), "Extraction", false);
		int labelingProtocolId = TestProtocol.test_create(SystemItems.getId(Protocol.LABELING), "Labeling", false);
		int poolingProtocolId = TestProtocol.test_create(SystemItems.getId(Protocol.POOLING), "Pooling", false);
		int hybProtocolId = TestProtocol.test_create(SystemItems.getId(Protocol.HYBRIDIZATION), "Hybridization", false);
		int hybStationId = TestHardware.test_create(SystemItems.getId(Hardware.HYBRIDIZATION_STATION), "Hybstation", false);
		int scanProtocolId = TestProtocol.test_create(SystemItems.getId(Protocol.SCANNING), "Scanning", false);
		int scannerId = TestHardware.test_create(SystemItems.getId(Hardware.SCANNER), "Scanner", false);
		int featureProtocolId = TestProtocol.test_create(SystemItems.getId(Protocol.FEATURE_EXTRACTION), "Feature", false);
		int featureSoftwareId = TestSoftware.test_create(SystemItems.getId(Software.FEATURE_EXTRACTION), "Feature", false);
		int genericId = TestPlatform.test_get(Platform.GENERIC);
		int variantId = TestPlatform.test_create_variant(genericId, "Variant", "test.variant", null, 0, null);
		int robotId = TestHardware.test_create(SystemItems.getId(Hardware.PRINT_ROBOT), "Robot", false);
		int printProtocolId = TestProtocol.test_create(SystemItems.getId(Protocol.PRINTING), "Printing", false);
		int freezerId = TestHardware.test_create(SystemItems.getId(Hardware.FREEZER), "Freezer", false);
		int childScanId = TestItemSubtype.test_create(Item.DERIVEDBIOASSAY, "ChildScan", SystemItems.getId(DerivedBioAssay.SCAN));
		int allSamplesSubtypeId = TestItemSubtype.test_create(Item.SAMPLE, "All samples");
		int base2DemoServer = TestFileServer.test_create("base.onk.lu.se", "BASE demo site", null, null, 
				"data/base.onk.lu.se.crt", null, null);

		TestProject.test_defaultItem(projectId, Item.PROTOCOL, defaultSamplingProtocolId);
		TestProject.test_defaultItem(projectId, Item.PROTOCOL, hybProtocolId);
		TestProject.test_defaultItem(projectId, Item.HARDWARE, hybStationId);
		
		// Import files
		int rootDirectoryId = TestDirectory.test_create(true, "file-import");
		int filesFileId = TestFile.test_create("data/test.batchimport.files.txt", false, false);
		int fileImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.FileImporter");
		int fileJobId = test_create_item_import_job(Item.FILE, fileImporterId, filesFileId, 0);
		ok = ok & TestJob.test_execute(fileJobId, false);
		ok = ok & TestDirectory.test_list("/file-import", 2); // 'raw' and 'ext'
		ok = ok & TestFile.test_list("/file-import", 1);  // 'reporters.txt'
		ok = ok & TestFile.test_list("/file-import/raw", 2);  // 'file1.txt' and 'file2.txt'
		ok = ok & TestFile.test_list("/file-import/ext", 2);  // 'robots.txt' and 'baselogo.png'
		
		// Import bioplates
		int bioPlateFileId = TestFile.test_create("data/test.batchimport.bioplates.txt", false, false);
		int bioPlateImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.BioPlateImporter");
		int bioPlateJobId = test_create_item_import_job(Item.BIOPLATE, bioPlateImporterId, bioPlateFileId, 0);
		ok = ok & TestJob.test_execute(bioPlateJobId, false);
		ok = ok & TestBioPlate.test_list(2);
		
		// Import biosources
		int bioSourceFileId = TestFile.test_create("data/test.batchimport.biosources.txt", false, false);
		int bioSourceImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.BioSourceImporter");
		int bioSourceJobId = test_create_item_import_job(Item.BIOSOURCE, bioSourceImporterId, bioSourceFileId, 0);
		ok = ok & TestJob.test_execute(bioSourceJobId, false);
		ok = ok & TestBioSource.test_list(3);
		
		// Import samples
		int sampleFileId = TestFile.test_create("data/test.batchimport.samples.txt", false, false);
		int sampleImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.SampleImporter");
		int sampleJobId = test_create_item_import_job(Item.SAMPLE, sampleImporterId, sampleFileId, allSamplesSubtypeId);
		ok = ok & TestJob.test_execute(sampleJobId, false);
		ok = ok & TestSample.test_list(4);

		// Import extracts
		int extractFileId = TestFile.test_create("data/test.batchimport.extracts.txt", false, false);
		int extractImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.ExtractImporter");
		int extractJobId = test_create_item_import_job(Item.EXTRACT, extractImporterId, extractFileId, 0);
		ok = ok & TestJob.test_execute(extractJobId, false);
		ok = ok & TestExtract.test_list(0, 4);

		// Import labeled extracts
		int labledExtractFileId = TestFile.test_create("data/test.batchimport.labeledextracts.txt", false, false);
		int labledExtractImporterId  = extractImporterId;
		int labledExtractJobId = test_create_item_import_job(Item.EXTRACT, labledExtractImporterId, labledExtractFileId, 0);
		ok = ok & TestJob.test_execute(labledExtractJobId, false);
		ok = ok & TestExtract.test_list(SystemItems.getId(Extract.LABELED), 7);
		
		// Import array designs
		int cdfId = TestFile.test_create("data/test.affymetrix.cdf", false, false);
		TestFile.test_rename(cdfId, "test.cdf");
		int mapId = TestFile.test_create("data/test.reportermap.import.txt", false, false);
		TestFile.test_rename(mapId, "test.map");
		int designFileId = TestFile.test_create("data/test.batchimport.arraydesigns.txt", false, false);
		int designImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.ArrayDesignImporter");
		int designJobId = test_create_item_import_job(Item.ARRAYDESIGN, designImporterId, designFileId, 0);
		ok = ok & TestJob.test_execute(designJobId, false);
		ok = ok & TestArrayDesign.test_list(3);
		
		// Import array batches
		int arrayBatchFileId = TestFile.test_create("data/test.batchimport.arraybatches.txt", false, false);
		int arrayBatchImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.ArrayBatchImporter");
		int arrayBatchJobId = test_create_item_import_job(Item.ARRAYBATCH, arrayBatchImporterId, arrayBatchFileId, 0);
		ok = ok & TestJob.test_execute(arrayBatchJobId, false);
		ok = ok & TestArrayBatch.test_list(3);

		// Import array slides
		int arraySlideFileId = TestFile.test_create("data/test.batchimport.arrayslides.txt", false, false);
		int arraySlideImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.ArraySlideImporter");
		int arraySlideJobId = test_create_item_import_job(Item.ARRAYSLIDE, arraySlideImporterId, arraySlideFileId, 0);
		ok = ok & TestJob.test_execute(arraySlideJobId, false);
		ok = ok & TestArraySlide.test_list(5);
		
		// Import hybridizations
		int hybFileId = TestFile.test_create("data/test.batchimport.hybridizations.txt", false, false);
		int hybImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.PhysicalBioAssayImporter");
		int hybJobId = test_create_item_import_job(Item.PHYSICALBIOASSAY, hybImporterId, hybFileId, 0);
		ok = ok & TestJob.test_execute(hybJobId, false);
		ok = ok & TestPhysicalBioAssay.test_list(3);

		// Import scans
		int samId = TestFile.test_create("data/test.assembly.sam", false, false);
		TestFile.test_rename(samId, "test.sam");
		int scanFileId = TestFile.test_create("data/test.batchimport.scans.txt", false, false);
		int scanImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.DerivedBioAssayImporter");
		int scanJobId = test_create_item_import_job(Item.DERIVEDBIOASSAY, scanImporterId, scanFileId, 0);
		ok = ok & TestJob.test_execute(scanJobId, false);
		ok = ok & TestDerivedBioAssay.test_list(0, 4);
	
		// Import raw bioassays
		int celId = TestFile.test_create("data/test.affymetrix.cel", false, false);
		TestFile.test_rename(celId, "test.cel");
		int gprId = TestFile.test_create("data/test.rawdata.import.txt", false, false);
		TestFile.test_rename(gprId, "test.gpr");
		int rbaFileId = TestFile.test_create("data/test.batchimport.rawbioassays.txt", false, false);
		int rbaImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.RawBioAssayImporter");
		int rbaJobId = test_create_item_import_job(Item.RAWBIOASSAY, rbaImporterId, rbaFileId, 0);
		ok = ok & TestJob.test_execute(rbaJobId, false);
		ok = ok & TestRawBioAssay.test_list(4);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();

		// Delete raw bioassays
		TestRawBioAssay.test_delete_all();
		TestJob.test_delete(rbaJobId);
		TestFile.test_delete(rbaFileId);
		TestFile.test_delete(celId);
		TestFile.test_delete(gprId);
		
		// Delete scans
		TestDerivedBioAssay.test_delete_all();
		TestJob.test_delete(scanJobId);
		TestFile.test_delete(scanFileId);
		TestFile.test_delete(samId);

		// Delete hybridizations
		TestPhysicalBioAssay.test_delete_all();
		TestJob.test_delete(hybJobId);
		TestFile.test_delete(hybFileId);

		// Delete array slides
		TestArraySlide.test_delete_all();
		TestJob.test_delete(arraySlideJobId);
		TestFile.test_delete(arraySlideFileId);
		
		// Delete array batches
		TestArrayBatch.test_delete_all();
		TestJob.test_delete(arrayBatchJobId);
		TestFile.test_delete(arrayBatchFileId);
		
		// Delete array designs
		TestArrayDesign.test_delete_all();
		TestJob.test_delete(designJobId);
		TestFile.test_delete(designFileId);
		TestFile.test_delete(cdfId);
		TestFile.test_delete(mapId);
		
		// Delete labeled extracts
		TestExtract.test_delete_all();
		TestJob.test_delete(labledExtractJobId);
		TestFile.test_delete(labledExtractFileId);

		// Delete extracts
		TestExtract.test_delete_all();
		TestJob.test_delete(extractJobId);
		TestFile.test_delete(extractFileId);

		// Delete samples
		TestSample.test_delete_all();
		TestJob.test_delete(sampleJobId);
		TestFile.test_delete(sampleFileId);

		// Delete biosources
		TestBioSource.test_delete_all();
		TestJob.test_delete(bioSourceJobId);
		TestFile.test_delete(bioSourceFileId);

		// Delete bioplates
		TestBioPlate.test_delete_all();
		TestJob.test_delete(bioPlateJobId);
		TestFile.test_delete(bioPlateFileId);
		
		// Delete files
		TestDirectory.test_delete(rootDirectoryId, true);
		TestJob.test_delete(fileJobId);
		TestFile.test_delete(filesFileId);
		
		// References items
		TestProtocol.test_delete(printProtocolId);
		TestHardware.test_delete(robotId);
		TestProtocol.test_delete(samplingProtocolId);
		TestProtocol.test_delete(defaultSamplingProtocolId);
		TestProtocol.test_delete(extractionProtocolId);
		TestProtocol.test_delete(labelingProtocolId);
		TestProtocol.test_delete(poolingProtocolId);
		TestProtocol.test_delete(hybProtocolId);
		TestHardware.test_delete(hybStationId);
		TestProtocol.test_delete(scanProtocolId);
		TestHardware.test_delete(scannerId);
		TestHardware.test_delete(freezerId);
		TestProtocol.test_delete(featureProtocolId);
		TestSoftware.test_delete(featureSoftwareId);
		TestPlatform.test_delete_variant(variantId);
		TestItemSubtype.test_delete(childScanId);
		TestItemSubtype.test_delete(allSamplesSubtypeId);
		TestFileServer.test_delete(base2DemoServer);
		TestProject.test_delete(projectId);

		write("++Testing item import using plugin "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create_item_import_job(Item itemType, int pluginDefinitionId, int fileId, int subtypeId)
	{
		if (pluginDefinitionId == 0 || fileId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition plugin = PluginDefinition.getById(dc, pluginDefinitionId);
			Job j = Job.getNew(dc, plugin, null, null);
			j.setName("Import " + itemType.toString());
			
			// Step 1 - file and parser regular expressions
			PluginConfigurationRequest request = j.configure(new GuiContext(itemType, GuiContext.Type.LIST));
			request.getRequestInformation();
			File file = File.getById(dc, fileId);
			request.setParameterValue("file", file);
			setParserParameters(dc, request, itemType);
			setColumnMappingParameters(request, itemType, file);
			if (subtypeId != 0)
			{
				request.setParameterValues("itemSubtypes", Collections.singletonList(ItemSubtype.getById(dc, subtypeId)));
			}
			
			
			PluginResponse response = request.invoke();
			if (response.getStatus() != Response.Status.CONTINUE)
			{
				Throwable t = response.getErrorList() != null ? response.getErrorList().get(0) : null;
				throw new BaseException(response.getMessage(), t);
			}

			// Step 2 - other options and error handling
			request = response.getNextRequest();
			request.getRequestInformation();
			setErrorHandlingParameters(request, itemType);
			response = request.invoke();
			
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for " + itemType + " import OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for " + itemType + " import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}


	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter<?> pp : ri.getParameters())
		{
			ParameterType pt = pp.getParameterType();
			System.out.println("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt );
		}
	}
	
	private static void setParserParameters(DbControl dc, PluginConfigurationRequest request, Item itemType)
	{
		request.setParameterValue("dataSplitterRegexp", "\\t");
		request.setParameterValue("dataHeaderRegexp", "Name.*");
		request.setParameterValue("ignoreRegexp", "#.*");
		
		if (itemType == Item.FILE)
		{
			request.setParameterValue("dataHeaderRegexp", "Path.*");
			request.setParameterValue("idMethod", "path");
			request.setParameterValue("root", Directory.getByPath(dc, new Path("/file-import", Path.Type.DIRECTORY)));
		}
		else
		{
			request.setParameterValue("idMethod", "name");
		}
		request.setParameterValue("mode", "create+update");
		request.setParameterValue("charset", "UTF-8");
	}
	
	private static void setColumnMappingParameters(PluginConfigurationRequest request, Item itemType, File file)
	{
		request.setParameterValue("nameColumnMapping", "\\Name\\");
		request.setParameterValue("descriptionColumnMapping", "\\Description\\");
		request.setParameterValue("complexExpressions", "allow");
		if (itemType == Item.FILE)
		{
			request.setParameterValue("pathColumnMapping", "\\Path\\");
			request.setParameterValue("urlColumnMapping", "\\URL\\");
			request.setParameterValue("fileServerColumnMapping", "\\File server\\");
			request.setParameterValue("mimeTypeColumnMapping", "\\Mime type\\");
			request.setParameterValue("subtypeColumnMapping", "\\Type\\");
		}
		if (itemType == Item.BIOPLATE)
		{
			request.setParameterValue("bioPlateTypeColumnMapping", "\\Bioplate type\\");
			request.setParameterValue("plateGeometryColumnMapping", "\\Plate geometry\\");
			request.setParameterValue("createdColumnMapping", "\\Created\\");
			request.setParameterValue("barcodeColumnMapping", "\\Barcode\\");
			request.setParameterValue("hardwareColumnMapping", "\\Freezer\\");
			request.setParameterValue("sectionColumnMapping", "\\Section\\");
			request.setParameterValue("trayColumnMapping", "\\Tray\\");
			request.setParameterValue("positionColumnMapping", "\\Position\\");
		}
		if (itemType == Item.SAMPLE || itemType == Item.EXTRACT)
		{
			request.setParameterValue("externalIdColumnMapping", "\\External ID\\");
			request.setParameterValue("protocolColumnMapping", "=default(col('Protocol'))");
			request.setParameterValue("createdColumnMapping", "\\Created\\");
			request.setParameterValue("originalQuantityColumnMapping", "\\Original quantity\\");
			request.setParameterValue("parentTypeColumnMapping", "\\Parent type\\");
			request.setParameterValue("parentColumnMapping", "\\Parent\\");
			request.setParameterValue("usedQuantityColumnMapping", "\\Used quantity\\");
		}
		if (file.getName().contains("labeledextracts"))
		{
			request.setParameterValue("tagColumnMapping", "\\Label\\");
			request.setParameterValue("parentTypeColumnMapping", "EXTRACT");
			request.setParameterValue("subtypeColumnMapping", "Labeled extract");
		}
		if (itemType == Item.PHYSICALBIOASSAY)
		{
			request.setParameterValue("subtypeColumnMapping", "Hybridization");
			request.setParameterValue("sizeColumnMapping", "\\Size\\");
			request.setParameterValue("protocolColumnMapping", "=default(col('Protocol'))");
			request.setParameterValue("hardwareColumnMapping", "=default()");
			request.setParameterValue("createdColumnMapping", "\\Created\\");
			request.setParameterValue("parentColumnMapping", "\\Parent\\");
			request.setParameterValue("usedQuantityColumnMapping", "\\Used quantity\\");
			request.setParameterValue("extractPositionColumnMapping", "\\Position\\");
			request.setParameterValue("arraySlideColumnMapping", "\\Array slide\\");
		}
		if (itemType == Item.DERIVEDBIOASSAY)
		{
			request.setParameterValue("dataDirectory", "/");
			request.setParameterValue("subtypeColumnMapping", "\\Subtype\\");
			request.setParameterValue("parentColumnMapping", "\\Parent\\");
			request.setParameterValue("extractColumnMapping", "\\Extract\\");
			request.setParameterValue("protocolColumnMapping", "\\Protocol\\");
			request.setParameterValue("hardwareColumnMapping", "\\Scanner\\");
			request.setParameterValue("fileTypeColumnMapping", "\\File type\\");
			request.setParameterValue("fileColumnMapping", "\\File\\");
		}
		if (itemType == Item.RAWBIOASSAY)
		{
			request.setParameterValue("dataDirectory", "/");
			request.setParameterValue("bioAssayColumnMapping", "\\Scan\\");
			request.setParameterValue("protocolColumnMapping", "\\Protocol\\");
			request.setParameterValue("platformColumnMapping", "\\Platform\\");
			request.setParameterValue("variantColumnMapping", "\\Variant\\");
			request.setParameterValue("rawDataTypeColumnMapping", "\\Raw data type\\");
			request.setParameterValue("softwareColumnMapping", "\\Software\\");
			request.setParameterValue("arrayDesignColumnMapping", "\\Array design\\");
			request.setParameterValue("fileTypeColumnMapping", "\\File type\\");
			request.setParameterValue("fileColumnMapping", "\\File\\");
		}
		if (itemType == Item.ARRAYDESIGN)
		{
			request.setParameterValue("dataDirectory", "/");
			request.setParameterValue("numArraysColumnMapping", "\\Num arrays\\");
			request.setParameterValue("platformColumnMapping", "\\Platform\\");
			request.setParameterValue("variantColumnMapping", "\\Variant\\");
			request.setParameterValue("fileTypeColumnMapping", "\\File type\\");
			request.setParameterValue("fileColumnMapping", "\\File\\");
		}
		if (itemType == Item.ARRAYBATCH)
		{
			request.setParameterValue("arrayDesignColumnMapping", "\\Array design\\");
			request.setParameterValue("protocolColumnMapping", "\\Protocol\\");
			request.setParameterValue("hardwareColumnMapping", "\\Robot\\");
		}
		if (itemType == Item.ARRAYSLIDE)
		{
			request.setParameterValue("arrayBatchColumnMapping", "\\Array batch\\");
			request.setParameterValue("indexColumnMapping", "\\Index\\");
			request.setParameterValue("barcodeColumnMapping", "\\Barcode\\");
			request.setParameterValue("destroyedColumnMapping", "\\Destroyed\\");
		}
	}
	
	private static void setErrorHandlingParameters(PluginConfigurationRequest request, Item itemType)
	{
		request.setParameterValue("defaultError", "fail");
		request.setParameterValue("stringTooLongError", "crop");
		request.setParameterValue("numberFormatError", "skip");
		request.setParameterValue("numberOutOfRangeError", "skip");

		request.setParameterValue("logFile", "/" + itemType.name() + ".log");
	}
	
}

