/**
	$Id: MergeTest.java 5827 2011-10-26 10:51:52Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.merge;

import java.sql.SQLException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.ArrayDesignBlock;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.BlockInfo;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicPositionQuery;
import net.sf.basedb.core.DynamicResultIterator;
import net.sf.basedb.core.DynamicSpotQuery;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.FeatureBatcher;
import net.sf.basedb.core.FeatureIdentificationMethod;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.IntensityTransform;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.MappingBatcher;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PositionBatcher;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataBatcher;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.Reporter;
import net.sf.basedb.core.ReporterBatcher;
import net.sf.basedb.core.SpotBatcher;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.Transformation;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.VirtualTable;
import net.sf.basedb.core.data.FeatureData;
import net.sf.basedb.core.data.RawData;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.JoinType;
import net.sf.basedb.core.query.Selects;
import net.sf.basedb.core.query.SqlResult;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.test.roles.Util;

/**
	This class contains the tests for setting up the merged experiment.
	<ol>
	</ol>

	@author nicklas
	@version 2.0
	@base.modified $Date: 2011-10-26 12:51:52 +0200 (on, 26 okt 2011) $
*/
public class MergeTest
{

	public static boolean ok = true;
	
	/**
		Run the tests in this class.
	*/
	public static boolean run()
	{
		DbControl dc = null;
		TestUtil.write("++Entering merge test\n");
		try
		{
			TestUtil.login("root", "root");
			
			// Reporters
			dc = TestUtil.getDbControl();
			createReporters(dc);
			dc.commit();
					
			// Array design
			dc = TestUtil.getDbControl();
			Platform generic = Platform.getByExternalId(dc, Platform.GENERIC);
			ArrayDesign design = createArrayDesign(dc, generic, "Array design for merge test");
			dc.commit();
			
			dc = TestUtil.getDbControl();
			createFeatures(dc, design);
			dc.commit();
			
			// Raw bioassays
			dc = TestUtil.getDbControl();
			RawDataType genepix = RawDataTypes.getSafeRawDataType("genepix");
			RawBioAssay rba1 = createRawBioAssay(dc, generic, genepix, "Raw bioassay for merge test #1", design);
			RawBioAssay rba2 = createRawBioAssay(dc, generic, genepix, "Raw bioassay for merge test #2", design);
			dc.commit();
			
			// Raw data
			dc = TestUtil.getDbControl();
			createRawData(dc, rba1);
			createRawData(dc, rba2);
			dc.commit();
			
			// Experiment
			dc = TestUtil.getDbControl();
			Experiment experiment = createExperiment(dc, "Experiment for merge test", rba1, rba2);
			dc.commit();

			// Root bioassayset's
			dc = TestUtil.getDbControl();
			Formula formula = Util.findFormula(dc, RawDataTypes.getSafeRawDataType("genepix"), Formula.Type.INTENSITY_EXPRESSION, "Mean FG");
			dc.commit();
			BioAssaySet root = createRootBioAssaySet(experiment, "Root", formula);
			
			// Merge on reporter
			dc = TestUtil.getDbControl();
			BioAssaySet mergedOnReporter = mergeOnReporter(dc, root);
			dc.commit();
			
			// Merge bioassays
			dc = TestUtil.getDbControl();
			BioAssaySet mergeOnBioAssays = mergeOnBioAssays(dc, root);
			dc.commit();
		
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving merge test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}
	
	
	public static void createReporters(DbControl dc)
	{
		TestUtil.write("--Creating reporters\n");
		ReporterBatcher batcher = ReporterBatcher.getNew(dc);
		for (int i = 0; i < 10; i++)
		{
			String externalId = "MERGE:" + i;
			if (!batcher.exists(externalId, false, true))
			{
				ReporterData reporter = new ReporterData();
				reporter.setName("Merge test #" + i);
				reporter.setExternalId(externalId);
				batcher.insert(reporter);
			}
		}
		batcher.flush();
		batcher.close();
	}
	
	/**
		Create an array design.
	*/
	public static ArrayDesign createArrayDesign(DbControl dc, Platform platform, String name)
	{
		TestUtil.write("--Creating array design: " + name + "\n");
		ArrayDesign design = ArrayDesign.getNew(dc, platform);
		design.setName(name);
		
		dc.saveItem(design);
		return design;
	}


	public static void createFeatures(DbControl dc, ArrayDesign design)
	{
		TestUtil.write("--Creating features for array design: " + design.getName() + "\n");
		dc.reattachItem(design, false);
		FeatureBatcher batcher = design.getFeatureBatcher(FeatureIdentificationMethod.COORDINATES, null);
		ReporterBatcher reporterBatcher = ReporterBatcher.getNew(dc);
		for (int block = 1; block <= 10; block++)
		{
			BlockInfo bi = new BlockInfo(block, 0 ,0);
			ArrayDesignBlock adb = design.addArrayDesignBlock(bi);
			for (int row = block; row <= 10; ++row)
			{
				String externalId = "MERGE:" + (row-1);
				ReporterData reporter = reporterBatcher.getByExternalId(externalId);
				FeatureData feature = batcher.newFeature(adb, reporter);
				feature.setRow(row);
				feature.setColumn(0);
				batcher.insert(feature);
			}
		}
		batcher.flush();
		batcher.close();
	}
	
	/**
		Create a raw bioassay.
	*/
	public static RawBioAssay createRawBioAssay(DbControl dc, Platform platform, RawDataType rawDataType, String name, ArrayDesign design)
	{
		TestUtil.write("--Creating raw bioassay: " + name + "\n");
		RawBioAssay rba = RawBioAssay.getNew(dc, platform, rawDataType);
		rba.setName(name);
		rba.setArrayDesign(design);
		dc.saveItem(rba);
		return rba;
	}
	
	public static void createRawData(DbControl dc, RawBioAssay rba)
	{
		TestUtil.write("--Creating data for raw bioassay: " + rba.getName() + "\n");
		dc.reattachItem(rba, false);
		RawDataBatcher batcher = rba.getRawDataBatcher(null, null);
		for (int block = 1; block <= 10; block++)
		{
			for (int row = block; row <= 10; ++row)
			{
				RawData rawData = batcher.newRawData();
				rawData.setBlock(block);
				rawData.setRow(row);
				rawData.setColumn(0);
				String externalId = "MERGE:" + (row-1);
				float ch1 = 200f + block * 10f - row * 10f;
				float ratio = row % 3 == 0 ? 0.5f : row % 3 == 1 ? 1f : 2f;
				float noise = 10 * (float)Math.random();
				rawData.setExtended("ch1FgMean", ch1 + noise);
				rawData.setExtended("ch2FgMean", ch1 * ratio - noise);
				batcher.insert(rawData, externalId, null);
			}
		}
		batcher.flush();
		batcher.close();
	}
	
	/**
		Create an experiment.
	*/
	public static Experiment createExperiment(DbControl dc, String name, RawBioAssay... rawBioAssays)
	{
		TestUtil.write("--Creating experiment: " + name + "\n");
		Experiment experiment = Experiment.getNew(dc, RawDataTypes.getSafeRawDataType("genepix"));
		experiment.setName(name);
		
		for (RawBioAssay rba : rawBioAssays)
		{
			rba = RawBioAssay.getById(dc, rba.getId()); // To get the latest information
			experiment.addRawBioAssay(rba);
			
			AnnotationSet as = rba.getAnnotationSet();
			for (Annotation a : as.getAllInheritedAnnotations().list(dc))
			{
				experiment.addExperimentalFactor(a.getAnnotationType());
			}
		}
		
		dc.saveItem(experiment);
		return experiment;
	}
		
	/**
		Create a root bioassay set.
	*/
	public static BioAssaySet createRootBioAssaySet(Experiment experiment, String name, Formula formula)
		throws Exception
	{
		TestUtil.write("--Creating root bioassay set: " + name + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.IntensityCalculatorPlugin");
			experiment = Experiment.getById(dc, experiment.getId());
			Job job = Job.getNew(dc, plugin, null, experiment);
			job.setName("Creating root bioassay set: " + name);
			job.setParameterValue("experiment", 
				new ItemParameterType<Experiment>(Experiment.class, null), experiment);
			job.setParameterValue("name", new StringParameterType(), name);
			job.setParameterValues("rawBioAssays", 
				new ItemParameterType<RawBioAssay>(RawBioAssay.class, null, true, 0, null), 
				experiment.getRawBioAssays().list(dc));
			job.setParameterValue("formula", 
				new ItemParameterType<Formula>(Formula.class, null), formula);
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
			
			dc = TestUtil.getDbControl();
			BioAssaySet root = Util.findBioAssaySet(dc, experiment, name);
			dc.close();
			return root;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	public static BioAssaySet mergeOnReporter(DbControl dc, BioAssaySet root)
		throws SQLException
	{
		TestUtil.write("--Merging on reporter\n");
		root = BioAssaySet.getById(dc, root.getId());
		Transformation t = root.newTransformation(null);
		t.setName("Merging on reporter");
		dc.saveItem(t);

		BioAssaySet child = t.newProduct("new", "new", false);
		child.setName("Merged on reporter");
		IntensityTransform transform = root.getIntensityTransform();
		child.setIntensityTransform(transform);
		dc.saveItem(child);
		
		Map<Short, Short> colMapping = new HashMap<Short, Short>();
		for (BioAssay ba : root.getBioAssays().list(dc))
		{
			BioAssay childBa = child.newBioAssay(Collections.singleton(ba));
			childBa.setName(ba.getName());
			dc.saveItem(childBa);
			colMapping.put(ba.getDataCubeColumnNo(), childBa.getDataCubeColumnNo());
		}
		
		// Get position->reporter mapping
		Map<Integer, Integer> posMapping = new HashMap<Integer, Integer>();
		DynamicPositionQuery posQuery = root.getPositionData();
		posQuery.select(Dynamic.select(VirtualColumn.REPORTER_ID));
		posQuery.setDistinct(true);
		PositionBatcher posBatcher = child.getPositionBatcher();
		int pos = 1;
		DynamicResultIterator result = posQuery.iterate(dc);
		while (result.hasNext())
		{
			int reporterId = result.next().getInt(1);
			posMapping.put(reporterId, pos);
			posBatcher.insert(pos, Reporter.getById(dc, reporterId));
			pos++;
		}
		posBatcher.flush();
		posBatcher.close();
		
		// Merge on reporter
		Formula.AverageMethod avgMethod = transform.getAverageMethod();
		SpotBatcher spotBatcher = child.getSpotBatcher();
		DynamicSpotQuery spotQuery = root.getSpotData();
		spotQuery.select(Dynamic.select(VirtualColumn.COLUMN));
		spotQuery.select(Dynamic.select(VirtualTable.POSITION, VirtualColumn.REPORTER_ID));
		spotQuery.select(Selects.expression(
			avgMethod.getAverageExpression(Dynamic.column(VirtualColumn.channelRaw(1))), "ch1"));
		spotQuery.select(Selects.expression(
			avgMethod.getAverageExpression(Dynamic.column(VirtualColumn.channelRaw(2))), "ch2"));
		spotQuery.group(Dynamic.column(VirtualColumn.COLUMN));
		spotQuery.group(Dynamic.column(VirtualTable.POSITION, VirtualColumn.REPORTER_ID));
		spotQuery.joinReporters(JoinType.INNER);
		
		result = spotQuery.iterate(dc);
		while (result.hasNext())
		{
			SqlResult data = result.next();
			short column = colMapping.get(data.getShort(1));
			int position = posMapping.get(data.getInt(2));
			float ch1 = data.getFloat(3);
			float ch2 = data.getFloat(4);
			spotBatcher.insert(column, position, ch1, ch2);
		}
		spotBatcher.flush();
		spotBatcher.close();
		
		// Map to raw data
		RawDataType rawDataType = root.getRawDataType();
		MappingBatcher mappingBatcher = child.getMappingBatcher();
		DynamicSpotQuery mapQuery = root.getSpotData();
		mapQuery.select(Dynamic.select(VirtualColumn.COLUMN));
		mapQuery.select(Dynamic.select(VirtualTable.POSITION, VirtualColumn.REPORTER_ID));
		mapQuery.select(Dynamic.select(VirtualTable.RAWPARENTS, VirtualColumn.RAWDATA_ID));
		mapQuery.joinReporters(JoinType.INNER);
		mapQuery.joinRawData(JoinType.INNER);
		result = mapQuery.iterate(dc);
		while (result.hasNext())
		{
			SqlResult data = result.next();
			short col = data.getShort(1);
			int position = posMapping.get(data.getInt(2));
			RawData rawData = rawDataType.getRawDataById(dc, data.getInt(3));
			mappingBatcher.insert(col, position, rawData);
		}
		mappingBatcher.flush();
		mappingBatcher.close();
		
		return child;
	}

	
	public static BioAssaySet mergeOnBioAssays(DbControl dc, BioAssaySet root)
		throws SQLException
	{
		TestUtil.write("--Merging on bioassays\n");
		root = BioAssaySet.getById(dc, root.getId());
		Transformation t = root.newTransformation(null);
		t.setName("Merging on bioassays");
		dc.saveItem(t);
	
		BioAssaySet child = t.newProduct("new", "new", false);
		child.setName("Merged on bioassay");
		IntensityTransform transform = root.getIntensityTransform();
		child.setIntensityTransform(transform);
		dc.saveItem(child);
		
		BioAssay childBa = child.newBioAssay(root.getBioAssays().list(dc));
		childBa.setName("Merged parent bioassays");
		dc.saveItem(childBa);
		
		// Get position->reporter mapping
		DynamicPositionQuery posQuery = root.getPositionData();
		posQuery.select(Dynamic.select(VirtualColumn.POSITION));
		posQuery.select(Dynamic.select(VirtualColumn.REPORTER_ID));
		PositionBatcher posBatcher = child.getPositionBatcher();
		posBatcher.insert(posQuery);
		posBatcher.flush();
		posBatcher.close();
		
		// Merge on bioassay
		Formula.AverageMethod avgMethod = transform.getAverageMethod();
		SpotBatcher spotBatcher = child.getSpotBatcher();
		DynamicSpotQuery spotQuery = root.getSpotData();
		spotQuery.select(Selects.expression(Expressions.integer(childBa.getDataCubeColumnNo()), "col"));
		spotQuery.select(Dynamic.select(VirtualColumn.POSITION));
		spotQuery.select(Selects.expression(
			avgMethod.getAverageExpression(Dynamic.column(VirtualColumn.channelRaw(1))), "ch1"));
		spotQuery.select(Selects.expression(
			avgMethod.getAverageExpression(Dynamic.column(VirtualColumn.channelRaw(2))), "ch2"));
		spotQuery.group(Dynamic.column(VirtualColumn.POSITION));
		spotBatcher.insert(spotQuery);
		spotBatcher.flush();
		spotBatcher.close();
		
		// Map to raw data
		MappingBatcher mappingBatcher = child.getMappingBatcher();
		DynamicSpotQuery mapQuery = root.getSpotData();
		mapQuery.select(Selects.expression(Expressions.integer(childBa.getDataCubeColumnNo()), "col"));
		mapQuery.select(Dynamic.select(VirtualColumn.POSITION));
		mapQuery.select(Dynamic.select(VirtualTable.RAWPARENTS, VirtualColumn.RAWDATA_ID));
		mapQuery.joinRawData(JoinType.INNER);
		mappingBatcher.insert(mapQuery);
		mappingBatcher.flush();
		mappingBatcher.close();
		
		return child;
	}
	
	
}
