/**
	$Id: RootTest.java 5827 2011-10-26 10:51:52Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.roles;

import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.Quota;
import net.sf.basedb.core.Role;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.User;
import net.sf.basedb.test.TestUtil;

/**
	This class contains the tests for the root user.
	<ol>
	<li>Create administrator: {@link #createAdmin(DbControl)}
	</ol>

	@author nicklas
	@version 2.0
	@base.modified $Date: 2011-10-26 12:51:52 +0200 (on, 26 okt 2011) $
*/
public class RootTest
{

	public static boolean ok = true;

	/**
		Run the tests in this class.
	*/
	public static boolean run()
	{
		DbControl dc = null;
		TestUtil.write("++Entering root user test\n");
		try
		{
			TestUtil.login("root", "root");
			dc = TestUtil.getDbControl();
			User admin = createAdmin(dc);
			dc.commit();
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving root user test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	
	/**
		Create user with administrator rights.
	*/
	public static User createAdmin(DbControl dc)
	{
		TestUtil.write("--Creating administrator\n");
		Quota unlimited = Quota.getById(dc, SystemItems.getId(Quota.UNRESTRICTED));
		Role adminRole = Role.getById(dc, SystemItems.getId(Role.ADMINISTRATOR));
		
		User admin = Util.createUser(dc, "Administrator", "admin", "admin", unlimited, null, 
				Collections.singleton(adminRole), null);
		Directory.createHomeDirectory(dc, admin, false);
		
		return admin;
	}
	
}
