/**
	$Id: UserTest.java 6021 2012-03-19 09:59:41Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.roles;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.BioMaterialEvent;
import net.sf.basedb.core.BioMaterialEventSource;
import net.sf.basedb.core.BioPlate;
import net.sf.basedb.core.BioPlateType;
import net.sf.basedb.core.BioSource;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DerivedBioAssay;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileParameterType;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.PhysicalBioAssay;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.PlateGeometry;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PlatformVariant;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.Project;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.Quantity;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.Sample;
import net.sf.basedb.core.Software;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.Tag;
import net.sf.basedb.core.Unit;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.test.FileUtil;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
	This class contains the tests for the regular user created
	by the root user.
	<ol>
	</ol>

	@author nicklas
	@version 2.0
	@base.modified $Date: 2012-03-19 10:59:41 +0100 (må, 19 mar 2012) $
*/
public class UserTest
{
	public static boolean ok = true;
	
	/**
		Run the tests in this class.
	*/
	public static boolean run(boolean useBatchImporters)
	{
		DbControl dc = null;
		TestUtil.write("++Entering user test\n");
		try
		{
			TestUtil.login("user", "user");
			PluginDefinition bioSourceBatchImporter = null;
			PluginDefinition sampleBatchImporter = null;
			PluginDefinition extractBatchImporter = null;
			PluginDefinition physicalBioAssayBatchImporter = null;
			PluginDefinition derivedBioAssayBatchImporter = null;
			PluginDefinition rawBioAssayBatchImporter = null;
			PluginDefinition annotationImporter = null;
			
			File batchImportFile_BioSource = null;
			File batchImportFile_Sample = null;
			File batchImportFile_Extract = null;
			File batchImportFile_PhysicalBioAssay = null;
			File batchImportFile_DerivedBioAssay = null;
			File batchImportFile_RawBioAssay = null; 
			

			// Activate project
			dc = TestUtil.getDbControl();
			Project p = Util.findProject(dc, "Project A");
			dc.close();
			TestUtil.getSessionControl().setActiveProject(p);
			
			if (useBatchImporters)
			{
				// Preparation: uploading files and find plugins
				dc = TestUtil.getDbControl();
				bioSourceBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.BioSourceImporter");
				sampleBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.SampleImporter");
				extractBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.ExtractImporter");
				physicalBioAssayBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.PhysicalBioAssayImporter");
				derivedBioAssayBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.DerivedBioAssayImporter");
				rawBioAssayBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.RawBioAssayImporter");
				annotationImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.AnnotationFlatFileImporter");
				
				batchImportFile_BioSource = FileUtil.uploadFile(dc, "/batchimport", "biosource_out.txt", null);
				batchImportFile_Sample = FileUtil.uploadFile(dc, "/batchimport", "sample_out.txt", null);
				batchImportFile_Extract = FileUtil.uploadFile(dc, "/batchimport", "extract_out.txt", null);
				batchImportFile_PhysicalBioAssay = FileUtil.uploadFile(dc, "/batchimport", "physicalbioassay_out.txt", null);
				batchImportFile_DerivedBioAssay = FileUtil.uploadFile(dc, "/batchimport", "derivedbioassay_out.txt", null);
				batchImportFile_RawBioAssay = FileUtil.uploadFile(dc, "/batchimport", "rawbioassay_out.txt", null);
				dc.commit();
			}

			// Create biomaterials
			dc = TestUtil.getDbControl();
			Quantity time = Quantity.getById(dc, SystemItems.getId(Quantity.TIME));
			Unit hoursUnit = time.getUnit("hour");
			Quantity potential = Quantity.getById(dc, SystemItems.getId(Quantity.POTENTIAL));
			Unit volt = potential.getUnit("V");

			Platform genericPlatform = Platform.getByExternalId(dc, Platform.GENERIC);
			Platform affymetrixPlatform = Platform.getByExternalId(dc, Platform.AFFYMETRIX);
			PlatformVariant seqPlatform = PlatformVariant.getByExternalId(dc, PlatformVariant.SEQUENCING_EXPRESSION);
			
			BioPlate bioPlate = createBioPlate(dc, "Bioplate A", Util.findGeometry(dc, 8, 12), 
					Util.findBioPlateType(dc, "Bioplate type A"));
			
			// Upload raw data files
			File rawData1 = FileUtil.uploadFile(dc, "/mouse", "genepix.mouse.v4.37k.00h.gpr", File.RAW_DATA);
			File rawData2 = FileUtil.uploadFile(dc, "/mouse", "genepix.mouse.v4.37k.24h.gpr", File.RAW_DATA);
			File rawData1DyeSwap = FileUtil.uploadFile(dc, "/mouse", "genepix.mouse.v4.37k.00h.dyeswap.gpr", File.RAW_DATA);
			File rawData2DyeSwap = FileUtil.uploadFile(dc, "/mouse", "genepix.mouse.v4.37k.24h.dyeswap.gpr", File.RAW_DATA);
			
			File affy1 = FileUtil.uploadFile(dc, "/affymetrix/E-TEST-1.ebi.ac.uk", "jos1761.cel.tar.bz2", File.RAW_DATA);
			File affy2 = FileUtil.uploadFile(dc, "/affymetrix/E-TEST-1.ebi.ac.uk", "jos1762.cel.tar.bz2", File.RAW_DATA);
			File affy3 = FileUtil.uploadFile(dc, "/affymetrix/E-TEST-1.ebi.ac.uk", "jos1763.cel.tar.bz2", File.RAW_DATA);
			
			File seqData1 = FileUtil.uploadFile(dc, "/sequencing", "dataset1_norm1/isoforms.fpkm_tracking.tar.bz2", File.RAW_DATA);
			File seqData2 = FileUtil.uploadFile(dc, "/sequencing", "dataset2_norm1/isoforms.fpkm_tracking.tar.bz2", File.RAW_DATA);
			dc.commit();
			
			BioSource bioSource = null;
			Sample s1 = null;
			Sample s2 = null;
			Sample sRef = null;
			
			Extract e1Qc = null;
			Extract e2Qc = null;
			Extract eRefQc = null;
			
			DerivedBioAssay sc1 = null;
			DerivedBioAssay sc2 = null;
			DerivedBioAssay sc1DyeSwap = null;
			DerivedBioAssay sc2DyeSwap = null;
			DerivedBioAssay affyScan1 = null;
			DerivedBioAssay affyScan2 = null;
			DerivedBioAssay affyScan3 = null;

			RawBioAssay rba1 = null;
			RawBioAssay rba2 = null;
			RawBioAssay rba1DyeSwap = null;
			RawBioAssay rba2DyeSwap = null;
			RawBioAssay affyRaw1 = null;
			RawBioAssay affyRaw2 = null;
			RawBioAssay affyRaw3 = null;
			RawBioAssay seqRaw1 = null;
			RawBioAssay seqRaw2 = null;
			
			if (!useBatchImporters)
			{
				dc = TestUtil.getDbControl();
				bioPlate = BioPlate.getById(dc, bioPlate.getId());
				bioSource =  createBioSource(dc, "Biosource A", "medium");
				s1 = createSample(dc, "Sample A.00h", bioSource, 0, hoursUnit);
				s1.setBioWell(bioPlate.getBioWell(0, 0));				
				s2 = createSample(dc, "Sample A.24h", bioSource, 24, hoursUnit);
				s2.setBioWell(bioPlate.getBioWell(0, 1));				
				sRef = createSample(dc, "Sample A.ref", null, null, null);
				sRef.setBioWell(bioPlate.getBioWell(0, 2));
				
				Extract e1 = createExtract(dc, "Extract A.00h", s1);
				e1.setBioWell(bioPlate.getBioWell(1, 0));
				Extract e2 = createExtract(dc, "Extract A.24h", s2);
				e2.setBioWell(bioPlate.getBioWell(1, 1));
				Extract eRef = createExtract(dc, "Extract A.ref", sRef);
				eRef.setBioWell(bioPlate.getBioWell(1, 2));
				
				// Quality control
				ItemSubtype qualityControl = Util.findItemSubtype(dc, Item.EXTRACT, "Quality control");
				AnnotationType rin = Util.findAnnotationType(dc, "RIN", Item.EXTRACT);
				e1Qc = createAliquot(dc, "Extract A.00h.qc", e1, qualityControl, null);
				e2Qc = createAliquot(dc, "Extract A.24h.qc", e2, qualityControl, null);
				eRefQc = createAliquot(dc, "Extract A.ref.qc", eRef, qualityControl, null);
				Util.setAnnotation(e1Qc, rin, 8.1f, null);
				Util.setAnnotation(e2Qc, rin, 8.5f, null);
				Util.setAnnotation(eRefQc, rin, 9.2f, null);
				
				Tag cy3 = Util.findTag(dc, "cy3");			
				Tag cy5 = Util.findTag(dc, "cy5");			
				Extract le1 = createLabeledExtract(dc, "Labeled extract A.00h", e1, cy3);
				le1.setBioWell(bioPlate.getBioWell(2, 0));
				Extract le1DyeSwap = createLabeledExtract(dc, "Labeled extract A.00h (dye-swap)", e1, cy5);
				le1DyeSwap.setBioWell(bioPlate.getBioWell(3, 0));
				Extract le2 = createLabeledExtract(dc, "Labeled extract A.24h", e2, cy3);
				le2.setBioWell(bioPlate.getBioWell(2, 1));
				Extract le2DyeSwap = createLabeledExtract(dc, "Labeled extract A.24h (dye-swap)", e2, cy5);
				le2DyeSwap.setBioWell(bioPlate.getBioWell(3, 1));
				Extract leRef = createLabeledExtract(dc, "Labeled extract A.ref", eRef, cy5);
				leRef.setBioWell(bioPlate.getBioWell(2, 2));
				Extract leRefDyeSwap = createLabeledExtract(dc, "Labeled extract A.ref (dye-swap)", eRef, cy3);
				leRefDyeSwap.setBioWell(bioPlate.getBioWell(3, 2));
				Extract lib1 = createLibrary(dc, "Library A.00h", e1);
				lib1.setBioWell(bioPlate.getBioWell(4, 0));
				Extract lib2 = createLibrary(dc, "Library A.24h", e2);
				lib2.setBioWell(bioPlate.getBioWell(4, 1));
				
				// Hybridizations, etc.
				PhysicalBioAssay h1 = createHybridization(dc, "Hybridization A.00h", "Array slide A.1", le1, leRef);
				PhysicalBioAssay h2 = createHybridization(dc, "Hybridization A.24h", "Array slide A.2", le2, leRef);
				PhysicalBioAssay h1DyeSwap = createHybridization(dc, "Hybridization A.00h (dye-swap)", "Array slide A.3", le1DyeSwap, leRefDyeSwap);
				PhysicalBioAssay h2DyeSwap = createHybridization(dc, "Hybridization A.24h (dye-swap)", "Array slide A.4", le2DyeSwap, leRefDyeSwap);
	
				// Affymetrix
				PhysicalBioAssay affyHyb1 = createHybridization(dc, "Affymetrix hyb A.1", "Affymetrix slide A.1", le1);
				PhysicalBioAssay affyHyb2 = createHybridization(dc, "Affymetrix hyb A.2", "Affymetrix slide A.2", le2);
				PhysicalBioAssay affyHyb3 = createHybridization(dc, "Affymetrix hyb A.3", "Affymetrix slide A.3", leRef);
				
				// Flow cell
				PhysicalBioAssay flowCell1 = createFlowCell(dc, "Flow cell A", 8, lib1, lib2);
				
				// Scans
				sc1 = createScan(dc, "Scan A.00h", h1, 400f, volt);
				sc2 = createScan(dc, "Scan A.24h", h2, 500f, volt);
				sc1DyeSwap = createScan(dc, "Scan A.00h (dye-swap)", h1DyeSwap, 600f, volt);
				sc2DyeSwap = createScan(dc, "Scan A.24h (dye-swap)", h2DyeSwap, 700f, volt);
	
				// Affymetrix scans
				affyScan1 = createScan(dc, "Affymetrix scan A.1", affyHyb1, 800f, volt);
				affyScan2 = createScan(dc, "Affymetrix scan A.2", affyHyb2, 900f, volt);
				affyScan3 = createScan(dc, "Affymetrix scan A.3", affyHyb3, 1000f, volt);
		
				// Sequence and alignment
				DerivedBioAssay sequencedAssay = createRootDerivedBioAssay(dc, "Sequenced A", flowCell1, 
						DerivedBioAssay.SEQUENCES, "HiSeq Settings A", "HiSeq 2000 A");
				DerivedBioAssay assembly1 = createDerivedBioAssay(dc, "Assembly A.00h", sequencedAssay, lib1,
						DerivedBioAssay.ARRANGEMENT, "TopHat Settings A", "TopHat");
				DerivedBioAssay assembly2 = createDerivedBioAssay(dc, "Assembly A.24h", sequencedAssay, lib2,
						DerivedBioAssay.ARRANGEMENT, "TopHat Settings A", "TopHat");
				
				// Raw bioassays
				ArrayDesign genericDesign = Util.findArrayDesign(dc, "Array design A");
				ArrayDesign affymetrixDesign = Util.findArrayDesign(dc, "Affymetrix A");
				ArrayDesign refSeqDesign = Util.findArrayDesign(dc, "RefSeqDesign A");
				RawDataType genepix = RawDataTypes.getSafeRawDataType("genepix");
				RawDataType cufflinksRaw = RawDataTypes.getSafeRawDataType("cufflinks");
				Protocol featureExtraction = Util.findProtocol(dc, "Feature extraction A");
				Protocol cufflinksSettings = Util.findProtocol(dc, "Cufflinks Settings A");
				Software softwareA = Util.findSoftware(dc, "Software A");
				Software cufflinks = Util.findSoftware(dc, "Cufflinks");
				
				rba1 = createRawBioAssay(dc, genericPlatform, genepix, "Raw bioassay A.00h", sc1, null, featureExtraction, softwareA, genericDesign);
				rba2 = createRawBioAssay(dc, genericPlatform, genepix, "Raw bioassay A.24h", sc2, null, featureExtraction, softwareA, genericDesign);
				rba1DyeSwap = createRawBioAssay(dc, genericPlatform, genepix, "Raw bioassay A.00h (dye-swap)", sc1DyeSwap, null, featureExtraction, softwareA, genericDesign);
				rba2DyeSwap = createRawBioAssay(dc, genericPlatform, genepix, "Raw bioassay A.24h (dye-swap)", sc2DyeSwap, null, featureExtraction, softwareA, genericDesign);
				
				// Affymetrix raw bioassays
				affyRaw1 = createRawBioAssay(dc, affymetrixPlatform, null, "Affymetrix raw A.1", affyScan1, null, featureExtraction, softwareA, affymetrixDesign);
				affyRaw2 = createRawBioAssay(dc, affymetrixPlatform, null, "Affymetrix raw A.2", affyScan2, null, featureExtraction, softwareA, affymetrixDesign);
				affyRaw3 = createRawBioAssay(dc, affymetrixPlatform, null, "Affymetrix raw A.3", affyScan3, null, featureExtraction, softwareA, affymetrixDesign);

				// Assign Affymetrix CEL files
				FileUtil.addDataFile(dc, affyRaw1, DataFileType.AFFYMETRIX_CEL, affy1, true);
				FileUtil.addDataFile(dc, affyRaw2, DataFileType.AFFYMETRIX_CEL, affy2, true);
				FileUtil.addDataFile(dc, affyRaw3, DataFileType.AFFYMETRIX_CEL, affy3, true);
				
				// Sequence raw bioassays
				seqRaw1 = createRawBioAssay(dc, seqPlatform, cufflinksRaw, "SeqRaw A.00h", assembly1, lib1, cufflinksSettings, cufflinks, refSeqDesign);
				seqRaw2 = createRawBioAssay(dc, seqPlatform, cufflinksRaw, "SeqRaw A.24h", assembly2, lib2, cufflinksSettings, cufflinks, refSeqDesign);
				
				FileUtil.addDataFile(dc, seqRaw1, DataFileType.FPKM_TRACKING, seqData1, true);
				FileUtil.addDataFile(dc, seqRaw2, DataFileType.FPKM_TRACKING, seqData2, true);
				
				dc.commit();
			}
			else
			{
				// Import biomaterials
				dc = TestUtil.getDbControl();
				importBioSources(bioSourceBatchImporter, batchImportFile_BioSource);
				importAnnotations(annotationImporter, batchImportFile_BioSource, Item.BIOSOURCE, getBioSourceColumnHeaders(), "\\t");
				
				importSamples(sampleBatchImporter, batchImportFile_Sample);
				importAnnotations(annotationImporter, batchImportFile_Sample, Item.SAMPLE, getSampleColumnHeaders(), "\\t");
				
				importExtracts(extractBatchImporter, batchImportFile_Extract);
				importAnnotations(annotationImporter, batchImportFile_Extract, Item.EXTRACT, getExtractColumnHeaders(), "\\t");
				
				// Hybridizations, etc.
				importPhysicalBioAssays(physicalBioAssayBatchImporter, batchImportFile_PhysicalBioAssay);			
				importDerivedBioAssays(derivedBioAssayBatchImporter, batchImportFile_DerivedBioAssay);
				importAnnotations(annotationImporter, batchImportFile_DerivedBioAssay, Item.DERIVEDBIOASSAY, getDerivedBioAssayColumnHeaders(), "\\t");
				importRawBioAssays(rawBioAssayBatchImporter, batchImportFile_RawBioAssay);
				bioSource = Util.findBioSource(dc, "Biosource A");
				s1 = Util.findSample(dc, "Sample A.00h");
				s2 = Util.findSample(dc, "Sample A.24h");
				sRef = Util.findSample(dc, "Sample A.ref");
				
				e1Qc = Util.findExtract(dc, "Extract A.00h.qc");
				e2Qc = Util.findExtract(dc, "Extract A.24h.qc");
				eRefQc = Util.findExtract(dc, "Extract A.ref.qc");
				
				sc1 = Util.findScan(dc, "Scan A.00h");
				sc2 = Util.findScan(dc, "Scan A.24h");
				sc1DyeSwap = Util.findScan(dc, "Scan A.00h (dye-swap)");
				sc2DyeSwap = Util.findScan(dc, "Scan A.24h (dye-swap)");
				affyScan1 = Util.findScan(dc, "Affymetrix scan A.1");
				affyScan2 = Util.findScan(dc, "Affymetrix scan A.2");
				affyScan3 = Util.findScan(dc, "Affymetrix scan A.3");
				dc.close();
			}
			dc = TestUtil.getDbControl();
			rba1 = Util.findRawBioAssay(dc, "Raw bioassay A.00h");
			rba2 = Util.findRawBioAssay(dc, "Raw bioassay A.24h");
			rba1DyeSwap = Util.findRawBioAssay(dc, "Raw bioassay A.00h (dye-swap)");
			rba2DyeSwap = Util.findRawBioAssay(dc, "Raw bioassay A.24h (dye-swap)");
			affyRaw1 = Util.findRawBioAssay(dc, "Affymetrix raw A.1");
			affyRaw2 = Util.findRawBioAssay(dc, "Affymetrix raw A.2");
			affyRaw3 = Util.findRawBioAssay(dc, "Affymetrix raw A.3");
			seqRaw1 = Util.findRawBioAssay(dc, "SeqRaw A.00h");
			seqRaw2 = Util.findRawBioAssay(dc, "SeqRaw A.24h");
			
			inheritAnnotations(dc, rba1, s1, e1Qc, bioSource, sc1);
			inheritAnnotations(dc, rba2, s2, e2Qc, bioSource, sc2);
			inheritAnnotations(dc, rba1DyeSwap, s1, e1Qc, bioSource, sc1DyeSwap);
			inheritAnnotations(dc, rba2DyeSwap, s2, e2Qc, bioSource, sc2DyeSwap);
			
			// Affymetrix
			inheritAnnotations(dc, affyRaw1, s1, e1Qc, bioSource, affyScan1);
			inheritAnnotations(dc, affyRaw2, s2, e2Qc, bioSource, affyScan2);
			inheritAnnotations(dc, affyRaw3, sRef, eRefQc, affyScan3);
			
			// Sequence
			inheritAnnotations(dc, seqRaw1, s1, e1Qc, bioSource);
			inheritAnnotations(dc, seqRaw2, s2, e2Qc, bioSource);
			
			PluginConfiguration rawDataImporter = Util.findPluginConfiguration(dc, "Raw data for project A");
			PluginConfiguration rawDataImporterDyeSwap = Util.findPluginConfiguration(dc, "Raw data for project A (dye-swap)");
			PluginConfiguration seqRawImporter = Util.findPluginConfiguration(dc, "Cufflinks isoform FPKM (transcript_id@seqname; no prefix)");
			dc.commit();
			
			// Import raw data
			importRawData(rawDataImporter, rawData1, rba1);
			importRawData(rawDataImporter, rawData2, rba2);
			importRawData(rawDataImporterDyeSwap, rawData1DyeSwap, rba1DyeSwap);
			importRawData(rawDataImporterDyeSwap, rawData2DyeSwap, rba2DyeSwap);
			
			importRawData(seqRawImporter, seqData1, seqRaw1);
			importRawData(seqRawImporter, seqData2, seqRaw2);
	
			// Experiment
			dc = TestUtil.getDbControl();
			Experiment exp1 = createExperiment(dc, "Experiment A", 
				RawDataTypes.getSafeRawDataType("genepix"), rba1, rba2, rba1DyeSwap, rba2DyeSwap);
			Experiment affyExperiment = createExperiment(dc, "Affymetrix A", 
				affymetrixPlatform.getRawDataType(), affyRaw1, affyRaw2, affyRaw3);
			Experiment seqExperiment = createExperiment(dc, "Sequence A", 
				RawDataTypes.getSafeRawDataType("cufflinks"), seqRaw1, seqRaw2);
			dc.commit();
			
			// Experiment overview and Analysis
			AnalysisTest.run("user");
			
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving user test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	/**
		Create a biosource.
	*/
	public static BioSource createBioSource(DbControl dc, String name, String drugResistance)
	{
		TestUtil.write("--Creating biosource: " + name + "\n");
		BioSource bioSource = BioSource.getNew(dc);
		bioSource.setName(name);
		dc.saveItem(bioSource);
		
		if (drugResistance != null)
		{
			AnnotationType at = Util.findAnnotationType(dc, "Drug resistance", Item.BIOSOURCE);
			Util.setAnnotation(bioSource, at, drugResistance, null);
		}
		
		return bioSource;
	}
	
	/**
		Create a sample.
	*/
	public static Sample createSample(DbControl dc, String name, BioSource bioSource, Integer time, Unit unit)
	{
		TestUtil.write("--Creating sample: " + name + "\n");
		Sample sample = Sample.getNew(dc);
		sample.setName(name);
		sample.setOriginalQuantity(100.0f);
		sample.getCreationEvent().setSource(bioSource);
		sample.getCreationEvent().setProtocol(Util.findProtocol(dc, "Sampling A"));
		dc.saveItem(sample);
		
		if (time != null)
		{
			AnnotationType at = Util.findAnnotationType(dc, "Time", Item.SAMPLE);
			Util.setAnnotation(sample, at, time, unit);
		}
		
		return sample;
	}
	
	/**
		Create an extract.
	*/
	public static Extract createExtract(DbControl dc, String name, Sample sample)
	{
		TestUtil.write("--Creating extract: " + name + "\n");
		Extract extract = Extract.getNew(dc);
		extract.setName(name);
		extract.setOriginalQuantity(200.0f);
		extract.getCreationEvent().setSource(sample).setUsedQuantity(50.0f);
		extract.getCreationEvent().setProtocol(Util.findProtocol(dc, "Extraction A"));
		dc.saveItem(extract);
		return extract;
	}

	/**
		Create a child extract.
	*/
	public static Extract createAliquot(DbControl dc, String name, Extract parent, ItemSubtype subtype, Protocol protocol)
	{
		TestUtil.write("--Creating aliquot: " + name + "\n");
		Extract extract = Extract.getNew(dc);
		extract.setName(name);
		extract.setItemSubtype(subtype);
		extract.setOriginalQuantity(2.0f);
		extract.getCreationEvent().setSource(parent).setUsedQuantity(2.0f);
		extract.getCreationEvent().setProtocol(protocol);
		dc.saveItem(extract);
		return extract;
	}
	
	/**
		Create a labeled extract.
	*/
	public static Extract createLabeledExtract(DbControl dc, String name, Extract extract, Tag label)
	{
		TestUtil.write("--Creating labeled extract: " + name + "\n");
		
		Extract labeledExtract = Extract.getNew(dc);
		labeledExtract.setItemSubtype(ItemSubtype.getById(dc, SystemItems.getId(Extract.LABELED)));
		labeledExtract.setName(name);
		labeledExtract.setOriginalQuantity(100.0f);
		labeledExtract.setTag(label);
		labeledExtract.getCreationEvent().setProtocol(Util.findProtocol(dc, "Labeling A"));
		labeledExtract.getCreationEvent().setSource(extract).setUsedQuantity(50.0f);
		dc.saveItem(labeledExtract);
		return labeledExtract;
	}

	/**
		Create a library.
	*/
	public static Extract createLibrary(DbControl dc, String name, Extract extract)
	{
		TestUtil.write("--Creating library: " + name + "\n");
		
		Extract library = Extract.getNew(dc);
		library.setItemSubtype(ItemSubtype.getById(dc, SystemItems.getId(Extract.LIBRARY)));
		library.setName(name);
		library.setOriginalQuantity(100.0f);
		library.getCreationEvent().setProtocol(Util.findProtocol(dc, "Library preparation A"));
		library.getCreationEvent().setSource(extract).setUsedQuantity(50.0f);
		dc.saveItem(library);
		return library;
	}

	
	/**
	 	Create a bioplate
	 */
	public static BioPlate createBioPlate(DbControl dc, String name, PlateGeometry geometry, BioPlateType plateType)
	{
		TestUtil.write("--Creating bioplate: " + name + "\n");
		BioPlate plate = BioPlate.getNew(dc, geometry, plateType);
		plate.setName(name);
		dc.saveItem(plate);
		return plate;
	}
	
	/**
		Create a hybridization.
	*/
	public static PhysicalBioAssay createHybridization(DbControl dc, String name, String arraySlideName, Extract... extracts)
	{
		TestUtil.write("--Creating hybridization: " + name + "\n");
		PhysicalBioAssay hyb = PhysicalBioAssay.getNew(dc);
		hyb.setItemSubtype(ItemSubtype.getById(dc, SystemItems.getId(PhysicalBioAssay.HYBRIDIZATION)));
		hyb.setName(name);
		hyb.setArraySlide(Util.findArraySlide(dc, arraySlideName));
		BioMaterialEvent creationEvent = hyb.getCreationEvent();
		creationEvent.setProtocol(Util.findProtocol(dc, "Hybridization A"));
		creationEvent.setHardware(Util.findHardware(dc, "Hybridization station A"));
		for (Extract le : extracts)
		{
			creationEvent.addSource(le).setUsedQuantity(50.0f);
		}
		dc.saveItem(hyb);
		return hyb;
	}

	/**
		Create a flow cell.
	*/
	public static PhysicalBioAssay createFlowCell(DbControl dc, String name, int size, Extract... extracts)
	{
		TestUtil.write("--Creating flow cell: " + name + "\n");
		PhysicalBioAssay flowCell = PhysicalBioAssay.getNew(dc);
		flowCell.setItemSubtype(ItemSubtype.getById(dc, SystemItems.getId(PhysicalBioAssay.CLONAL_AMPLIFICATION)));
		flowCell.setName(name);
		flowCell.setSize(size);
		
		BioMaterialEvent creationEvent = flowCell.getCreationEvent();
		creationEvent.setProtocol(Util.findProtocol(dc, "cBot Settings A"));
		creationEvent.setHardware(Util.findHardware(dc, "cBot A"));
		int pos = 1;
		for (Extract lib : extracts)
		{
			BioMaterialEventSource src = creationEvent.addSource(lib);
			src.setUsedQuantity(50.0f);
			src.setPosition(pos);
			pos++;
		}
		dc.saveItem(flowCell);
		return flowCell;
	}

	/**
		Create a scan.
	*/
	public static DerivedBioAssay createScan(DbControl dc, String name, PhysicalBioAssay hyb, Float pmtGain, Unit unit)
	{
		TestUtil.write("--Creating scan: " + name + "\n");
		DerivedBioAssay scan = DerivedBioAssay.getNew(dc, hyb, null);
		scan.setItemSubtype(ItemSubtype.getById(dc, SystemItems.getId(DerivedBioAssay.SCAN)));
		scan.setName(name);
		scan.setProtocol(Util.findProtocol(dc, "Scanning A"));
		scan.setHardware(Util.findHardware(dc, "Scanner A"));
		dc.saveItem(scan);
		
		if (pmtGain != null)
		{
			AnnotationType at = Util.findAnnotationType(dc, "PMT gain", Item.DERIVEDBIOASSAY);
			Util.setAnnotation(scan, at, pmtGain, unit);
		}

		return scan;
	}

	/**
		Create a root derived bioassay.
	*/
	public static DerivedBioAssay createRootDerivedBioAssay(DbControl dc, String name, PhysicalBioAssay parent, 
		String subtype, String protocol, String hardware)
	{
		TestUtil.write("--Creating root bioassay: " + name + "\n");
		DerivedBioAssay bioAssay = DerivedBioAssay.getNew(dc, parent, null);
		bioAssay.setItemSubtype(ItemSubtype.getById(dc, SystemItems.getId(subtype)));
		bioAssay.setName(name);
		bioAssay.setProtocol(Util.findProtocol(dc, protocol));
		bioAssay.setHardware(Util.findHardware(dc, hardware));
		dc.saveItem(bioAssay);
	
		return bioAssay;
	}

	/**
		Create a child derived bioassay.
	*/
	public static DerivedBioAssay createDerivedBioAssay(DbControl dc, String name, DerivedBioAssay parent, 
		Extract extract, String subtype, String protocol, String software)
	{
		TestUtil.write("--Creating derived bioassay: " + name + "\n");
		DerivedBioAssay bioAssay = DerivedBioAssay.getNew(dc, parent, null);
		bioAssay.setItemSubtype(ItemSubtype.getById(dc, SystemItems.getId(subtype)));
		bioAssay.setName(name);
		bioAssay.setProtocol(Util.findProtocol(dc, protocol));
		bioAssay.setSoftware(Util.findSoftware(dc, software));
		bioAssay.setExtract(extract);
		dc.saveItem(bioAssay);
	
		return bioAssay;
	}

	
	/**
		Create a raw bioassay.
	*/
	public static RawBioAssay createRawBioAssay(DbControl dc, Platform platform, 
		RawDataType rawDataType, String name, DerivedBioAssay bioAssay, Extract extract,
		Protocol protocol, Software software, ArrayDesign design)
	{
		TestUtil.write("--Creating raw bioassay: " + name + "\n");
		RawBioAssay rba = RawBioAssay.getNew(dc, platform, rawDataType);
		rba.setName(name);
		rba.setParentBioAssay(bioAssay);
		rba.setParentExtract(extract);
		rba.setProtocol(protocol);
		rba.setSoftware(software);
		rba.setArrayDesign(design);
		dc.saveItem(rba);
		return rba;
	}

	/**
		Create a raw bioassay.
	*/
	public static RawBioAssay createRawBioAssay(DbControl dc, PlatformVariant variant, 
		RawDataType rawDataType, String name, DerivedBioAssay bioAssay, Extract extract,
		Protocol protocol, Software software, ArrayDesign design)
	{
		TestUtil.write("--Creating raw bioassay: " + name + "\n");
		RawBioAssay rba = RawBioAssay.getNew(dc, variant, rawDataType);
		rba.setName(name);
		rba.setParentBioAssay(bioAssay);
		rba.setParentExtract(extract);
		rba.setProtocol(protocol);
		rba.setSoftware(software);
		rba.setArrayDesign(design);
		dc.saveItem(rba);
		return rba;
	}
	
	/**
		Inherit all annotations from the parent.
	*/
	public static void inheritAnnotations(DbControl dc, Annotatable item, Annotatable... parents)
	{
		AnnotationSet as = item.getAnnotationSet();
		for (Annotatable parent : parents)
		{
			parent = (Annotatable)parent.getType().getById(dc, parent.getId());
			if (parent.isAnnotated())
			{
				as.inheritAnnotationSet(parent.getAnnotationSet());
			}
		}
	}
	
	/**
		Import features.
	*/
	public static void importRawData(PluginConfiguration config, File file, RawBioAssay rba)
		throws Exception
	{
		TestUtil.write("--Importing raw data from file: " + file.getName() + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			config = PluginConfiguration.getById(dc, config.getId());
			Job job = Job.getNew(dc, config.getPluginDefinition(), config, null);
			job.setName("Importing data to raw bioassay: " + rba.getName());
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("rawBioAssay", 
				new ItemParameterType<RawBioAssay>(RawBioAssay.class, null), rba);
			job.setParameterValue("missingReporterError", new StringParameterType(), "null");
			job.setParameterValue("featureMismatchError", new StringParameterType(), "smart");
			job.setParameterValue("numberFormatError", new StringParameterType(), "null");
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	/**
		Create an experiment.
	*/
	public static Experiment createExperiment(DbControl dc, String name, RawDataType rawDataType, RawBioAssay... rawBioAssays)
	{
		TestUtil.write("--Creating experiment: " + name + "\n");
		Experiment experiment = Experiment.getNew(dc, rawDataType);
		experiment.setName(name);
		
		for (RawBioAssay rba : rawBioAssays)
		{
			rba = RawBioAssay.getById(dc, rba.getId()); // To get the latest information
			experiment.addRawBioAssay(rba);
			
			AnnotationSet as = rba.getAnnotationSet();
			for (Annotation a : as.getAllInheritedAnnotations().list(dc))
			{
				experiment.addExperimentalFactor(a.getAnnotationType());
			}
			for (Annotation a : as.getAnnotations().list(dc))
			{
				experiment.addExperimentalFactor(a.getAnnotationType());
			}
		}
		
		dc.saveItem(experiment);
		return experiment;
	}

	public static void importRawBioAssays(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing raw bioassays from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of raw bioassays to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tPlatform\\tVariant\\tRaw data type\\tParent\\tExtract\\tSoftware\\tProtocol\\tArray design\\tDescription\\tFile type\\tFile");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("rawDataTypeColumnMapping", new StringParameterType(), "\\Raw data type\\");
			job.setParameterValue("platformColumnMapping", new StringParameterType(), "\\Platform\\");
			job.setParameterValue("variantColumnMapping", new StringParameterType(), "\\Variant\\");
			job.setParameterValue("bioAssayColumnMapping", new StringParameterType(), "\\Parent\\");
			job.setParameterValue("extractColumnMapping", new StringParameterType(), "\\Extract\\");
			job.setParameterValue("softwareColumnMapping", new StringParameterType(), "\\Software\\");
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			job.setParameterValue("arrayDesignColumnMapping", new StringParameterType(), "\\Array design\\");
			job.setParameterValue("fileTypeColumnMapping", new StringParameterType(), "\\File type\\");
			job.setParameterValue("fileColumnMapping", new StringParameterType(), "\\File\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	public static void importDerivedBioAssays(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing derived bioassays from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of derived bioassays to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tSubtype\\tParent type\\tParent\\tExtract\\tDescription\\tProtocol\\tHardware\\tSoftware\\tPMT gain");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("subtypeColumnMapping", new StringParameterType(), "\\Subtype\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("parentTypeColumnMapping", new StringParameterType(), "\\Parent type\\");
			job.setParameterValue("parentColumnMapping", new StringParameterType(), "\\Parent\\");
			job.setParameterValue("extractColumnMapping", new StringParameterType(), "\\Extract\\");
			job.setParameterValue("hardwareColumnMapping", new StringParameterType(), "\\Hardware\\");			
			job.setParameterValue("softwareColumnMapping", new StringParameterType(), "\\Software\\");			
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	public static void importPhysicalBioAssays(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing physical bioassays from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of physical bioassays to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tSubtype\\tArray slide\\tDescription\\tHardware\\tSize\\tProtocol\\tParent\\tParent used quantity\\tParent position\\tCreated");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("subtypeColumnMapping", new StringParameterType(), "\\Subtype\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("hardwareColumnMapping", new StringParameterType(), "\\Hardware\\");
			job.setParameterValue("parentColumnMapping", new StringParameterType(), "\\Parent\\");
			job.setParameterValue("usedQuantityColumnMapping", new StringParameterType(), "\\Parent used quantity\\");
			job.setParameterValue("extractPositionColumnMapping", new StringParameterType(), "\\Parent position\\");
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			job.setParameterValue("arraySlideColumnMapping", new StringParameterType(), "\\Array slide\\");
			job.setParameterValue("sizeColumnMapping", new StringParameterType(), "\\Size\\");
			job.setParameterValue("createdColumnMapping", new StringParameterType(), "\\Created\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	public  static void importExtracts(PluginDefinition batchImporter,	File file)
	{
		TestUtil.write("--Importing extracts from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of extracts to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tSubtype\\tTag\\tOriginal quantity\\tDescription\\tExternal id\\tProtocol\\tCreated\\tParent type\\tParent\\tParent used quantity\\tBioplate\\tRow\\tColumn\\tRIN");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("subtypeColumnMapping", new StringParameterType(), "\\Subtype\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("externalIdColumnMapping", new StringParameterType(), "\\External id\\");
			job.setParameterValue("originalQuantityColumnMapping", new StringParameterType(), "\\Original quantity\\");
			job.setParameterValue("createdColumnMapping", new StringParameterType(), "\\Created\\");
			job.setParameterValue("parentTypeColumnMapping", new StringParameterType(), "\\Parent type\\");
			job.setParameterValue("parentColumnMapping", new StringParameterType(), "\\Parent\\");
			job.setParameterValue("tagColumnMapping", new StringParameterType(), "\\Tag\\");
			job.setParameterValue("usedQuantityColumnMapping", new StringParameterType(), "\\Parent used quantity\\");
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			job.setParameterValue("bioPlateColumnMapping", new StringParameterType(), "\\Bioplate\\");
			job.setParameterValue("bioWellRowColumnMapping", new StringParameterType(), "\\Row\\");
			job.setParameterValue("bioWellColColumnMapping", new StringParameterType(), "\\Column\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	public static void importSamples(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing samples from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of samples to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tOriginal quantity\\tDescription\\tExternal id\\tCreated\\tBiosource\\tProtocol\\tBioplate\\tRow\\tColumn\\tTime");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("externalIdColumnMapping", new StringParameterType(), "\\External id\\");
			job.setParameterValue("originalQuantityColumnMapping", new StringParameterType(), "\\Original quantity\\");
			job.setParameterValue("createdColumnMapping", new StringParameterType(), "\\Created\\");
			job.setParameterValue("parentColumnMapping", new StringParameterType(), "\\Biosource\\");
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			job.setParameterValue("bioPlateColumnMapping", new StringParameterType(), "\\Bioplate\\");
			job.setParameterValue("bioWellRowColumnMapping", new StringParameterType(), "\\Row\\");
			job.setParameterValue("bioWellColColumnMapping", new StringParameterType(), "\\Column\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if(dc != null) dc.close();
		}		
	}

	public static void importBioSources(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing biosources from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of bio-sources to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), "Name\\tDescription\\tExternal id\\tDrug resistance");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("externalIdColumnMapping", new StringParameterType(), "\\External id\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if(dc != null) dc.close();
		}		
	}
	
	public static void importAnnotations(PluginDefinition annotationImporter, File file, Item item, String[] columnHeaders, String dataSplitter)
	{
		TestUtil.write("--Importing annotations for " + item.name() + " from file: " + file + "\n");
		DbControl dc = null;
		StringBuilder headerString = new StringBuilder();
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, annotationImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			job.setName("Batch import of annotations for " + item.name());
			
			// Get annotation types for the current item
			ItemQuery<AnnotationType> query = AnnotationType.getQuery(item);
			query.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
			query.order(Orders.asc(Hql.property("name")));
			List<AnnotationType> annotationTypes;
			Map<String, AnnotationType> names = 
				new TreeMap<String, AnnotationType>(String.CASE_INSENSITIVE_ORDER);
			
			annotationTypes = new ArrayList<AnnotationType>(query.list(dc));
			for (AnnotationType at : annotationTypes)
			{
				names.put(at.getName(), at);
			}
			for (int i=0; i<columnHeaders.length; i++)
			{
				String header = columnHeaders[i];
				headerString.append(i>0 ? "\\t" : "");
				headerString.append(header);
				if (names.get(header) != null)
				{
					job.setParameterValue("columnMapping." + i, 
							new ItemParameterType<AnnotationType>(AnnotationType.class, annotationTypes), 
							names.get(header));
				}
			}
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), dataSplitter);
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), headerString.toString());
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("itemType", new StringParameterType(), item.name());
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null ) dc.close();
		}
	}
	
	private static String[] getBioSourceColumnHeaders()
	{
		return new String[] {"Name", "Description", "External id", "Drug resistance"};
	}
	
	private static String[] getSampleColumnHeaders()
	{
		return new String[] {"Name", "Original quantity", "Description", "External id", "Created", "Biosource", "Protocol", "Bioplate", "Row", "Column", "Time"};
	}
	
	private static String[] getExtractColumnHeaders()
	{
		return new String[] {"Name", "Subtype", "Tag", "Original quantity", "Description", "External id", "Protocol", "Created", "Parent type", "Parent", "Parent used quantity", "Bioplate", "Row", "Column", "RIN"};
	}

	private static String[] getDerivedBioAssayColumnHeaders()
	{
		return new String[] {"Name", "Subtype", "Parent type", "Parent", "Extract", "Description", "Protocol", "Hardware", "Software", "PMT gain"};
	}

}
