/**
	$Id: AbstractRPCClient.java 6434 2014-03-14 08:30:20Z nicklas $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.client;

import java.io.IOException;
import java.io.InputStream;

import javax.activation.DataHandler;
import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMText;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.rpc.client.RPCServiceClient;

/**
 	Abstract class for RPCServiceClients in web services.
 	Holds information about the service to use
 	and gives different ways of  	
  	@author Martin
  	@version 2.5
 	@deprecated In 3.3, no replacement
 */
@Deprecated
public abstract class AbstractRPCClient 
{
	private final String serviceUrl;
	private ServiceFactory serviceFactory;
	
	protected AbstractRPCClient(String url, String service)
	{
		this(url, service, null);
	}
	
	/**
		
		@since 2.8
	*/
	protected AbstractRPCClient(String url, String service, ServiceFactory serviceFactory)
	{
		this.serviceUrl = url + "/" + service;
		this.serviceFactory = serviceFactory == null ? 
			Factories.getDefaultServiceFactory() : serviceFactory;
	}
	
	/**
		Get the factory that is used to create {@link RPCServiceClient}
		objects for this web service.
		@since 2.8
	*/
	public ServiceFactory getServiceFactory()
	{
		return serviceFactory;
	}
	
	/**
		Sets the factory that is used to create {@link RPCServiceClient}
		objects for this web service.
		@since 2.8
	*/
	public void setServiceFactory(ServiceFactory serviceFactory)
	{
		this.serviceFactory = serviceFactory;
	}
	
	protected Object[] invokeBlocking(String operation, Object[] args, Class[] returnTypes)
		throws AxisFault
	{
        return getService().invokeBlocking(getOperation(operation), args, returnTypes);
	}
	
	protected <T> T invokeBlocking(String method, Class<T> clazz, Object... args)
		throws AxisFault
	{
		Object[] result = invokeBlocking(method, args, new Class[] { clazz });
		return clazz == null ? null : clazz.cast(result[0]);
	}
	
	protected void invokeBlocking(String operation, Object... args)
		throws AxisFault
	{
		getService().invokeRobust(getOperation(operation), args);
	}
	
	protected InputStream invokeFileBlocking(String operation, Object... args)
		throws AxisFault, IOException
	{
		OMElement response = getService().invokeBlocking(getOperation(operation), args);
		OMElement fileElement = response.getFirstElement();
		OMElement dataElement = fileElement.getFirstElement();	
		if (dataElement == null) return null;
		OMText node = (OMText) dataElement.getFirstOMChild();
		node.setBinary(true);
		DataHandler dataHandler = (DataHandler) node.getDataHandler();		
		return dataHandler.getInputStream();
	}
	
	protected RPCServiceClient getService()
		throws AxisFault
	{
		EndpointReference target = new EndpointReference(serviceUrl);
		RPCServiceClient service = serviceFactory.createService(target);
		Options options = service.getOptions();
		if (options.getTo() == null) options.setTo(target);
		return service;
	}
	
	protected QName getOperation(String operation)
	{
		return new QName("http://server.ws.basedb.sf.net", operation);
	}
	
}
