/**
	$Id: BioAssaySetClient.java 6435 2014-03-17 08:01:38Z nicklas $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.client;

import java.io.IOException;
import java.io.InputStream;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.client.Options;
import org.apache.axis2.rpc.client.RPCServiceClient;

/**
 	Web service client for BioAssaySetService in BASE.
 	{@link net.sf.basedb.ws.server.BioAssaySetService}
  	@author Martin
  	@version 2.5
 	@deprecated In 3.3, no replacement
 */
@Deprecated
public class BioAssaySetClient 
	extends AbstractRPCClient
{
	private final SessionClient session;
	
	/**
	 	Creates new BioAssayClient for a session.
	  	@param session Active session.
	 */
	public BioAssaySetClient(SessionClient session)
	{
		super(session.getUrl(), "BioAssaySet", session.getServiceFactory());
		this.session = session;
	}
	
	/**
	 	Returns if a bioassay set has spot data in the form of files that 
	 	can be downloaded or not.
	  	@param bioAssaySetId Id of the bioassay set it concerns.
	  	@return TRUE if there are data to be downloaded, FALSE otherwise.
	  	@throws AxisFault If the communication with the webservice fails.
	  	@since 2.12
	 */
	public Boolean hasDownloadableData(int bioAssaySetId) 
		throws AxisFault
	{
		return invokeBlocking
			(
				"hasDownloadableData",
				Boolean.class,
				session.getId(),
				bioAssaySetId
			);
	}

	/**
	 	Returns information about data file types that are used storing
	 	spot data for a bioassay set.
	  	@param bioAssaySetId Id of the bioassay set to get the information about.
	  	@param qOpt Restrictions and includes to put on the query when getting the information.
	  	@return An array of DataFileTypeInfo, one for each data file type used in the raw bioassay.
	  	@throws AxisFault If communication with webservice fails.
	  	@since 2.12
	*/
	public net.sf.basedb.info.DataFileTypeInfo[] getDataFileTypes(int bioAssaySetId, net.sf.basedb.info.QueryOptions qOpt)
		throws AxisFault
	{
		return invokeBlocking
			(
				"getDataFileTypes",
				net.sf.basedb.info.DataFileTypeInfo[].class,
				session.getId(),
				bioAssaySetId,
				qOpt
			);
	}

	/**
	 	Returns information about data files that are used storing
	 	data for a bioassay set
	  	@param bioAssaySetId Id of the bioassay set to get the information about.
	  	@param qOpt Restrictions and includes to put on the query when getting the information.
	  	@return An array of FileSetMemberInfo, one for each data file used in the bioassay set.
	  	@throws AxisFault If communication with webservice fails.
	  	@since 3.0
	*/
	public net.sf.basedb.info.FileSetMemberInfo[] getDataFiles(int bioAssaySetId, net.sf.basedb.info.QueryOptions qOpt)
		throws AxisFault
	{
		return invokeBlocking
				(
					"getDataFiles",
					net.sf.basedb.info.FileSetMemberInfo[].class,
					session.getId(),
					bioAssaySetId,
					qOpt
				);
	}

	
	/**
	 	Gets spot data as a file for a bioassay set if it is stored in a file
	 	in BASE.
	  	@param bioAssaySetId Id of bioassay set to get the file from
	  	@param dataFileType The external ID of the data file type to download
	  	@return An InputStream to read the file data from, or null
	  		if no file of the specified type exists
	  	@throws AxisFault If communication with webservices fails.
	  	@throws IOException If writing to the target file fails.
	  	@since 2.12
	*/
	public InputStream xdownloadDataFile(int bioAssaySetId, String dataFileType) 
		throws AxisFault, IOException
	{
		RPCServiceClient service = getService();
		Options options = service.getOptions();		
		options.setProperty(Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		service.setOptions(options);
		return invokeFileBlocking
			(
				"downloadDataFile", 
				session.getId(), 
				bioAssaySetId,
				dataFileType
			);
	}

	
}
