/**
	$Id: ArrayDesignService.java 6435 2014-03-17 08:01:38Z nicklas $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.server;


import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.FileSet;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.SessionControl;

import org.apache.axiom.om.OMElement;

/**
 	Web service class for 
 	{@link net.sf.basedb.core.ArrayDesign}
 	Provides services to access limited
 	functionality in this class.
  	@author Martin
 	@version 2.5
 	@deprecated In 3.3, no replacement
 */
@Deprecated
public class ArrayDesignService 
	extends AbstractRPCService
{
	/**
	 	
	 */
	public ArrayDesignService()
	{}
	
	/**
		Get information about an array design that you know the ID of.
		@param ID Id to identify the session.
		@param itemId The ID of the array design
		@return The array design information
	*/
	public net.sf.basedb.info.ArrayDesignInfo getById(String ID, int itemId)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();	
		try
		{
			ArrayDesign ad = ArrayDesign.getById(dc, itemId);
			return ad.toTransferable(new net.sf.basedb.info.ArrayDesignInfo());
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
	/**
	 	Gets an array with info about array designs 
	 	available for the active session.
	  	@param ID Id for the session to use. 
	  	@param qOpt QueryOptions to put on the query
	  	@return An ArrayDesignInfo array.
	 */
	public net.sf.basedb.info.ArrayDesignInfo[] getArrayDesigns(String ID, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{	
			ItemQuery<ArrayDesign> query= ArrayDesign.getQuery();		
			query = util.getConfiguredItemQuery(query, qOpt);
			return util.listToInfo(query.list(dc), net.sf.basedb.info.ArrayDesignInfo.class);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	/**
	 	Returns array design file of a certain type, attached to an OMElement.
	 	This method assumes that only a single file of the given type exists. If there 
	 	are two or more files, then it is not specified which one that is returned. 
	 	Use {@link #getDataFiles(String, int, QueryOptions)} and
	 	{@link FileService} to handle the multi-file case.
	 	
	  	@param ID Used to identify an active session with.
	  	@param arrayDesignId Id of the array design the file is associated with.
	  	@param dataFileType IThe external ID of the file type.
	  	@return OMElement with the file attached, 
	  		or NULL if no file was found.
	 */
	public OMElement downloadArrayDesignByType(String ID,
				int arrayDesignId, String dataFileType)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		OMElement toDownload = null;
		try
		{
			ArrayDesign design = ArrayDesign.getById(dc, arrayDesignId);
			if (design.hasFileSet())
			{
				FileSet fileSet = design.getFileSet();
				DataFileType type = DataFileType.getByExternalId(dc, dataFileType);
				if (fileSet.hasMember(type))
				{
					FileSetMember member = fileSet.getMember(type);
					BaseFileDataSource source = new BaseFileDataSource(member.getFile());
					toDownload = attachFile(source, "downloadFile");
				}
			}
		}
		finally
		{
			if (dc != null) dc.close();
		}		
		return toDownload;
	}
	
	/**
	 	Gets if the arraydesign files can be downloaded or not.
	  	@param ID Used to identify an active session with
	  	@param arrayDesignId Id of the array design it concerns
	  	@return TRUE if there are files to download, FALSE otherwise.
	 */
	public Boolean hasDownloadableData(String ID, int arrayDesignId)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		ArrayDesign design = null;
		try
		{
			design = ArrayDesign.getById(dc, arrayDesignId);
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return design != null ? design.hasFileSet() : false;
	}
	
	/**
	 	Returns information objects for each DataFileType represented in the FileSet
	 	of an ArrayDesign
	  	@param ID Id of the active session.
	  	@param arrayDesignId Id of the array design it concerns
	 	@param qOpt Restrictions and includes to put on the query
	  	@return An array with DataFileTypeInfo:s.
	 */
	public net.sf.basedb.info.DataFileTypeInfo[] getDataFileTypes(String ID, int arrayDesignId, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		
		try
		{
			ArrayDesign design = ArrayDesign.getById(dc, arrayDesignId);
			if (design.hasFileSet())
			{
				FileSet fileSet = design.getFileSet();
				ItemQuery<DataFileType> query = fileSet.getMemberTypes();				
				query = util.getConfiguredItemQuery(query, qOpt);
				return util.listToInfo(query.list(dc), net.sf.basedb.info.DataFileTypeInfo.class);
			}
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return new net.sf.basedb.info.DataFileTypeInfo[0];
	}
	
	/**
	 	Returns information objects for each data file that is associated with 
	 	an Array design.
	  	@param ID Id of the active session.
	  	@param arrayDesignId Id of the array design it concerns
	 	@param qOpt Restrictions and includes to put on the query
	  	@return An array with FileSetMemberInfo
	  	@since 3.0
	*/
	public net.sf.basedb.info.FileSetMemberInfo[] getDataFiles(String ID, int arrayDesignId, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{
			ArrayDesign design = ArrayDesign.getById(dc, arrayDesignId);
			return util.getDataFiles(dc, design, qOpt);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	/**
		Get the annotation values for one or more annotation types.
		@param ID Id of the active session.
		@param arrayDesignId Id of the array design it concerns
		@param annotationTypes Array with the ID of the annotation types
		@param findInherited TRUE to look among inherited annotations, FALSE to only look
			at primary annotations
		@return An array with AnnotationInfo object, each element in the array 
			corresponds to the annotation type with the same index in the annotationTypes array
	*/
	public net.sf.basedb.info.AnnotationInfo[] getAnnotations(String ID, int arrayDesignId, Integer[] annotationTypes, boolean findInherited)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{
			ArrayDesign design = ArrayDesign.getById(dc, arrayDesignId);
			return util.getAnnotations(dc, design, annotationTypes, findInherited);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
}
