/**
	$Id: BioAssaySetService.java 6435 2014-03-17 08:01:38Z nicklas $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.server;

import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.FileSet;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.SessionControl;

import org.apache.axiom.om.OMElement;

/**
 	Web service for BioAssaySet {@link net.sf.basedb.core.BioAssaySet}.
 	This class provides services to get information and data from BioAssaySet.
  	@author Martin
  	@version 2.5
 	@deprecated In 3.3, no replacement
 */
@Deprecated
public class BioAssaySetService 
	extends AbstractRPCService
{	
	/**
	 	
	 */
	public BioAssaySetService()
	{}
	
	/**
	 	Returns a data file of a certain type attached to an OMElement.
	 	This method assumes that only a single file of the given type exists. If there 
	 	are two or more files, then it is not specified which one that is returned. 
	 	Use {@link #getDataFiles(String, int, QueryOptions)} and
	 	{@link FileService} to handle the multi-file case.

	  	@param ID Used to identify an active session with.
	  	@param bioAssaySetId Id of the bioassay set the file belongs to.
	  	@param dataFileType The external ID of the file type.
	  	@return OMElement with the file attached, or NULL if no file was found.
	  	@see DataFileType
	  	@since 2.12
	*/
	public OMElement downloadDataFile(String ID, int bioAssaySetId, String dataFileType)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		OMElement toDownload = null;
		try
		{
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			if (bas.hasFileSet())
			{
				FileSet fileSet = bas.getFileSet();
				DataFileType type = DataFileType.getByExternalId(dc, dataFileType);
				if (fileSet.hasMember(type))
				{
					FileSetMember member = fileSet.getMember(type);
					BaseFileDataSource source = new BaseFileDataSource(member.getFile());
					toDownload = attachFile(source, "downloadFile");
				}
			}
		}
		finally
		{
			if (dc != null) dc.close();
		}		
		return toDownload;
		
	}
	
	/**
	 	Checks if the files has been attached to bioassay set.
	  	@param ID Used to identify an active session with
	  	@param bioAssaySetId Id of the bioassay set it concerns
	  	@return TRUE if there are files to download, FALSE otherwise.
	  	@since 2.12
	*/
	public Boolean hasDownloadableData(String ID, int bioAssaySetId)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		BioAssaySet bas = null;
		try
		{
			bas = BioAssaySet.getById(dc, bioAssaySetId);
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return bas != null ? bas.hasFileSet() : false;
	}

	/**
	 	Returns information objects for each DataFileType represented in the FileSet
	 	of a BioAssaySet.
	  	@param ID Id of the active session.
	  	@param bioAssaySetId Id of the bioassay set it concerns
	 	@param qOpt Restrictions and includes to put on the query
	  	@return An array with DataFileTypeInfo:s.
	  	@since 2.12
	*/
	public net.sf.basedb.info.DataFileTypeInfo[] getDataFileTypes(String ID, int bioAssaySetId, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			if (bas.hasFileSet())
			{			
				ItemQuery<DataFileType> query = bas.getFileSet().getMemberTypes();
				query = util.getConfiguredItemQuery(query, qOpt);
				return util.listToInfo(query.list(dc), net.sf.basedb.info.DataFileTypeInfo.class);
			}
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return new net.sf.basedb.info.DataFileTypeInfo[0];
	}

	/**
	 	Returns information objects for each data file that is associated with 
	 	a bioassay set.
	  	@param ID Id of the active session.
	  	@param bioAssaySetId Id of the bioassay set it concerns
	 	@param qOpt Restrictions and includes to put on the query
	  	@return An array with FileSetMemberInfo
	  	@since 3.0
	*/
	public net.sf.basedb.info.FileSetMemberInfo[] getDataFiles(String ID, int bioAssaySetId, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			return util.getDataFiles(dc, bas, qOpt);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	/**
		Get the annotation values for one or more annotation types.
		@param ID Id of the active session.
		@param bioAssaySetId Id of the bioassay set it concerns
		@param annotationTypes Array with the ID of the annotation types
		@param findInherited TRUE to look among inherited annotations, FALSE to only look
			at primary annotations
		@return An array with AnnotationInfo object, each element in the array 
			corresponds to the annotation type with the same index in the annotationTypes array
	*/
	public net.sf.basedb.info.AnnotationInfo[] getAnnotations(String ID, int bioAssaySetId, Integer[] annotationTypes, boolean findInherited)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			return util.getAnnotations(dc, bas, annotationTypes, findInherited);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}	
	
}