/**
	$Id: ExperimentService.java 6435 2014-03-17 08:01:38Z nicklas $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.server;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.ReporterList;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	Web Service class for 
	{@link net.sf.basedb.core.Experiment}
  	@author Nicklas, Martin
  	@version 2.5
 	@deprecated In 3.3, no replacement
 */
@Deprecated
public class ExperimentService
	extends AbstractRPCService
{
	/**
	 	
	 */
	public ExperimentService()
	{}
	
	/**
		Get information about an experiment that you know the ID of.
		@param ID Id to identify the session.
		@param itemId The ID of the experiment
		@return The experiment information
	*/
	public net.sf.basedb.info.ExperimentInfo getById(String ID, int itemId)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();	
		try
		{
			Experiment e = Experiment.getById(dc, itemId);
			return e.toTransferable(new net.sf.basedb.info.ExperimentInfo());
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
	/**
	 	Gets an array with info about experiments 
	 	available for the active session.
	  	@param ID Id for the session to use. 
	  	@param qOpt QueryOptions to put on the query that gets the Experiments.
	  	@return An ExperimentInfo array.
	 */
	public net.sf.basedb.info.ExperimentInfo[] getExperiments(String ID, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{	
			ItemQuery<Experiment> query= Experiment.getQuery();		
			query = util.getConfiguredItemQuery(query, qOpt);
			return util.listToInfo(query.list(dc), net.sf.basedb.info.ExperimentInfo.class);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	/**
	 	Gets information about the bioassay sets in an experiment. The caller
	 	of this method can put restriction on the query that gets the bioassay sets.
	  	@param ID Id to identify the active session with
	  	@param experimentId Id of the experiment the bioassay sets are in.
	  	@param qOpt Includes and restrictions to put on query that gets the bioassay sets.
	  	@return Array of ItemInfo:s, one for each bioassay set that where found by the query.
	 */
	public net.sf.basedb.info.BioAssaySetInfo[] getBioAssaySets(String ID, int experimentId, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{
			Experiment experiment = Experiment.getById(dc, experimentId);
			ItemQuery<BioAssaySet> query = experiment.getBioAssaySets();
			query = util.getConfiguredItemQuery(query, qOpt);
			return util.listToInfo(query.list(dc), net.sf.basedb.info.BioAssaySetInfo.class);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	/**
	 	Gets an array with information objects for each
	 	rawbioassay in an experiment.
	  	@param ID The id to identify an active session.
	  	@param experimentId The id of the experiment to get the rawbioassays from.
	  	@param qOpt QueryOptions to put on the query when getting the rawbioassays.
	  	@return Array with {@link net.sf.basedb.info.RawBioAssayInfo}s
	 */
	public net.sf.basedb.info.RawBioAssayInfo[] getRawBioAssays(String ID, int experimentId, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{
			Experiment experiment = Experiment.getById(dc, experimentId);
			ItemQuery<RawBioAssay> query = experiment.getRawBioAssays();			
			query = util.getConfiguredItemQuery(query, qOpt);
			return util.listToInfo(query.list(dc), net.sf.basedb.info.RawBioAssayInfo.class);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}	
	
	/**
	 	Gets an array with information objects for each reporter list used 
	 	in an Experiment. 
	  	@param ID The id to identify an active session.
	  	@param experimentId The id of the experiment to get the reporter lists from.
	  	@param qOpt QueryOptions to put on the query when getting the reporter lists.
	  	@return ReporterListInfo array.
	 */
	public net.sf.basedb.info.ReporterListInfo[] getReporterLists(String ID, int experimentId, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{
			Experiment experiment = Experiment.getById(dc, experimentId);
			ItemQuery<ReporterList> query = ReporterList.getQuery();	
			query.restrictPermanent(Restrictions.eq(Hql.property("experiment"), Hql.entity(experiment)));
			return util.listToInfo(query.list(dc), net.sf.basedb.info.ReporterListInfo.class);
		}
		finally
		{
			if (dc != null) dc.close();
		}		
	}

	/**
	 	Gets an array with information objects for each
	 	experimental factor in an experiment.
	  	@param ID The id to identify an active session.
	  	@param experimentId The id of the experiment to get the rawbioassays from.
	  	@param qOpt QueryOptions to put on the query
	  	@return Array with {@link net.sf.basedb.info.AnnotationTypeInfo}:s
	*/
	public net.sf.basedb.info.AnnotationTypeInfo[] getExperimentalFactors(String ID, int experimentId, net.sf.basedb.info.QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();
		try
		{
			Experiment experiment = Experiment.getById(dc, experimentId);
			ItemQuery<AnnotationType> query = experiment.getExperimentalFactors();			
			query = util.getConfiguredItemQuery(query, qOpt);
			return util.listToInfo(query.list(dc), net.sf.basedb.info.AnnotationTypeInfo.class);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}	
}
