/**
	$Id: TempFileDataSource.java 6434 2014-03-14 08:30:20Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.server;

import java.io.File;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.activation.FileDataSource;

/**
	A DataSource implementation for temporary files. The temporary
	file will be deleted once the entire file has been read or the
	close() method is called on the InputStream. 
	<p>
	Since this may cause problems if the {@link #getInputStream()}
	is called more than once, the file deletion is disabled if
	that happens. Instead the {@link File#deleteOnExit()} is 
	called to mark the file for deletion when the virtual machine exits.

	@author nicklas
	@version 2.5
	@base.modified $Date: 2014-03-14 09:30:20 +0100 (fr, 14 mar 2014) $
 	@deprecated In 3.3, no replacement
 */
@Deprecated
public class TempFileDataSource
	extends FileDataSource
{
	private final String mimeType;
	private int numStreams = 0;
	
	public TempFileDataSource(File file, String mimeType)
	{
		super(file);
		this.mimeType = mimeType;
	}
	
	/**
		Get the supplied MIME type, if any. Otherwise call
		super.getContentType().
	*/
	@Override
	public String getContentType()
	{
		return mimeType != null ? mimeType : super.getContentType();
	}
	
	/**
		Get an InputStream that deletes the source file as soon
		as the end of file is reached or close() is called. If 
		this method is called more than once the file deletion is
		disabled since it may interfere with other streams.
	*/
	@Override
	public InputStream getInputStream()
		throws IOException
	{
		numStreams++;
		return new TempInputStream(super.getInputStream(), getFile());
	}
	
	/**
		An InputStream implementation that deletes the source file
		once the entire stream has been read (=any of the read()
		methods return -1) or when close() is called.
	*/
	private class TempInputStream
		extends FilterInputStream
	{
		private File file;
		private TempInputStream(InputStream in, File file)
		{
			super(in);
			this.file = file;
		}

		@Override
		public int read()
			throws IOException
		{
			int result = super.read();
			if (result == -1) deleteTempFile();
			return result;
		}
		
		@Override
		public int read(byte[] b)
			throws IOException
		{
			int result = super.read(b);
			if (result == -1) deleteTempFile();
			return result;
		}
		
		@Override
		public int read(byte[] b, int off, int len)
			throws IOException
		{
			int result = super.read(b, off, len);
			if (result == -1) deleteTempFile();
			return result;
		}
		
		@Override
		public void close()
			throws IOException
		{
			super.close();
			deleteTempFile();
		}
		
		@Override
		protected void finalize()
			throws Throwable
		{
			super.finalize();
			deleteTempFile();
		}
		
		/**
	 		Delete the temp file if it exists and we know that
	 		only one stream is reading from it.
		*/
		private void deleteTempFile()
		{
			if (file != null && file.exists()) 
			{
				if (numStreams ==1)
				{
					file.delete();
					file = null;
				}
				else
				{
					file.deleteOnExit();
				}
			}
		}
	}
	
}
