/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Software = function()
{
	var software = {};
	
	/**
		Initialize the page.
	*/
	software.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', software.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Subtyp list
			Events.addEventHandler('subtype_id', 'change', software.subtypeOnChange);
			software.subtypeOnChange();
			
			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', software.loadAnnotationsFrame);
			TabControl.addTabValidator('settings.info', software.validateSoftware);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'SOFTWARE', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);

			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'SOFTWARE'};
			var tableAttributes = {'table-id': 'software'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwnerOfItems', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}

	// Validate the software information
	software.validateSoftware = function()
	{
		var frm = document.forms['software'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}
	
	// Save the software settings
	software.save = function()
	{
		var frm = document.forms['software'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			frm.submit();
		}
	}

	software.loadAnnotationsFrame = function()
	{
		Annotations.autoLoadEditFrame(null, ItemSubtype.getSubtypeId('subtype_id'));
	}

	// Get default software of selected subtype for the current project (if any)
	software.subtypeOnChange = function()
	{
		var frm = document.forms['software'];
		if (frm.replaceExistingDefault)
		{
			var subtypeId = ItemSubtype.getSubtypeId('subtype_id');
			var defaultInfo = ItemSubtype.getProjectDefaultAndRecentItems('SOFTWARE', subtypeId);
			var currentDefaultDiv = Doc.element('currentDefault');
				
			var tooltip = '';
			var html = '';
			var checked = true;
			var disabled = false;
				
			var numDefaults = defaultInfo['default'] ? defaultInfo['default'].length : 0;
			if (numDefaults == 1)
			{
				html = '('+Strings.encodeTags(Strings.cut(defaultInfo['default'][0].name, 40)) + ')';
			}
			else if (numDefaults == 0)
			{
				html = '(-none-)';
				checked = false;
				disabled = true;
			}
			else
			{
				html = '(' + numDefaults + ' different)';
				for (var i = 0; i < numDefaults; i++)
				{
					tooltip += Strings.encodeTags(defaultInfo['default'][i].name) + '\n';
				}
				checked = false;
			}
			
			currentDefaultDiv.title = tooltip;
			currentDefaultDiv.innerHTML = html;
			frm.replaceExistingDefault.checked = checked;
			frm.replaceExistingDefault.disabled = disabled;
		}
	}
	
	return software;
}();

Doc.onLoad(Software.initPage);
