/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Footnote = function()
{
	var pAutoReload = 60; // seconds until automatic page reload
	var footnote = {};
	
	/**
		Initialize the page.
	*/
	footnote.initPage = function()
	{
		// Broadcast message popup
		Buttons.addClickHandler('broadcast', footnote.showBroadcastMessage);
		Buttons.addClickHandler('singleMessage', footnote.singleMessageOnClick)
		Buttons.addClickHandler('multiMessages', footnote.showAllMessages);
		
		Footnote.loadMessages();
	}
	
	/**
		Reload the footnote page.
	*/
	footnote.loadMessages = function()
	{
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetFootnoteMessages';
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, footnote.messagesLoaded, footnote.messagesNotLoaded);
		request.send(null);
	}
	
	footnote.messagesLoaded = function(request)
	{
		var messages = JSON.parse(request.responseText);
		
		Doc.hide('broadcast');
		Doc.hide('credit');
		Doc.hide('singleMessage');
		Doc.hide('multiMessages');
		if (messages.broadcast)
		{
			Doc.show('broadcast');
			Doc.element('broadcast-title').innerHTML = Strings.encodeTags(messages.broadcast);
		}
		else if (messages.newMessages > 1)
		{
			Doc.show('multiMessages');
			Doc.element('num-messages').innerHTML = messages.newMessages;
		}
		else if (messages.newMessages == 1)
		{
			var msg = messages.message;
			Doc.show('singleMessage');
			Data.set('singleMessage', 'message-id', msg.id);
			Doc.element('message-name').innerHTML = Strings.encodeTags(msg.name);
			Doc.element('message-from').innerHTML = Strings.encodeTags(msg.from);
		}
		else
		{
			Doc.show('credit');
		}
		setTimeout(footnote.loadMessages, pAutoReload * 1000);
	}

	footnote.messagesNotLoaded = function(request)
	{
		App.debug('Footnote not updated: ' + request.responseText);
		setTimeout(footnote.loadMessages, pAutoReload * 1000);
	}
	
	/**
		Show a popup with the broadcast message.
	*/
	footnote.showBroadcastMessage = function()
	{
		Dialogs.openPopup(App.getRoot()+'info/broadcast.jsp', 'BroadcastMessage', 450, 300);
	}
	
	/**
		'click' handler for showing a single message in a popup
	*/
	footnote.singleMessageOnClick = function(event)
	{
		var messageId = Data.get(event.currentTarget, 'message-id');
		if (!messageId) return;
		Items.viewItem('MESSAGE', messageId);
		Footnote.startReloadTimer(1);
	}
	
	/**
		Show all messages in the main frame.
	*/
	footnote.showAllMessages = function()
	{
		window.top['main'].location.href = App.getRoot()+'my_base/messages/index.jsp?ID='+App.getSessionId();
	}
	
	return footnote;
}();

Doc.onLoad(Footnote.initPage);